Package: wjake
Title: Personal Themes and Formatting Preferences
Version: 1.0.1
Authors@R: 
    person(given = "W. Jake",
           family = "Thompson",
           role = c("aut", "cre", "cph"),
           email = "wjakethompson@gmail.com",
           comment = c(ORCID = "0000-0001-7339-0300"))
Description: A collection of utility functions, themes, and templates to
    support personal data analysis workflows. Includes functions for
    formatting numeric values as text, custom themes and color scales for
    'ggplot2', and automatic formatting for tables created with 'gt'.
License: MIT + file LICENSE
URL: https://github.com/wjakethompson/wjake,
        https://wjake.wjakethompson.com
BugReports: https://github.com/wjakethompson/wjake/issues
Depends: R (>= 4.5.0)
Imports: cli, colorspace, dplyr, english, ggplot2 (>= 4.0.0), ggtext,
        gh, glue, grDevices, gt, knitr, pak, rlang (>= 1.1.0),
        rstudioapi, scales, showtextdb, stringr, sysfonts, systemfonts,
        xfun
Suggests: spelling, taylor, testthat, vdiffr, withr
Config/testthat/edition: 3
Config/Needs/website: wjakethompson/wjaketemplate
Config/Needs/check: wjakethompson/wjaketemplate
Config/Needs/coverage: wjakethompson/wjaketemplate
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-06-09 18:48:57 UTC; jakethompson
Author: W. Jake Thompson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7339-0300>)
Maintainer: W. Jake Thompson <wjakethompson@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-09 19:00:02 UTC
Built: R 4.5.2; ; 2026-06-09 21:54:48 UTC; unix

# Generated by roxygen2: do not edit by hand

export(announce)
export(apa_words)
export(code_heading)
export(fmt_corr)
export(fmt_count)
export(fmt_digits)
export(fmt_pct)
export(fmt_prop)
export(fmt_prop_pct)
export(gt_theme_apa)
export(gt_theme_wjake)
export(make_color_pal)
export(palette_okabeito)
export(palette_wjake)
export(ramp_blue)
export(ramp_orgblu)
export(ramp_yelblu)
export(ramp_yellow)
export(round_to)
export(scale_color_okabeito)
export(scale_color_wjake)
export(scale_colour_okabeito)
export(scale_colour_wjake)
export(scale_fill_okabeito)
export(scale_fill_wjake)
export(scale_palette)
export(scale_x_comma)
export(scale_x_percent)
export(scale_y_comma)
export(scale_y_percent)
export(theme_wjake)
export(write_pkg_bib)
import(ggplot2)
import(rlang)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# wjake <a href="https://wjake.wjakethompson.com"><img src="man/figures/logo.png" align="right" height="139" alt="wjake website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![lifecycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![R package
version](https://www.r-pkg.org/badges/version/wjake)](https://CRAN.R-project.org/package=wjake)
[![Package
downloads](https://cranlogs.r-pkg.org/badges/grand-total/wjake)](https://cran.r-project.org/package=wjake)</br>
[![R-CMD-check](https://github.com/wjakethompson/wjake/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/wjakethompson/wjake/actions/workflows/R-CMD-check.yaml)
[![codecov](https://codecov.io/gh/wjakethompson/wjake/branch/main/graph/badge.svg?token=i9e5Zt18sa)](https://app.codecov.io/gh/wjakethompson/wjake)
[![pages-build-deployment](https://github.com/wjakethompson/wjake/actions/workflows/pages/pages-build-deployment/badge.svg)](https://github.com/wjakethompson/wjake/actions/workflows/pages/pages-build-deployment)</br>
[![Signed
by](https://img.shields.io/badge/Keybase-Verified-brightgreen.svg)](https://keybase.io/wjakethompson)
![License](https://img.shields.io/badge/License-MIT-blue.svg)
<!-- badges: end -->

This is a personal R package containing utility functions and custom
themes for [ggplot2](https://ggplot2.tidyverse.org/) and
[gt](https://gt.rstudio.com/).

## Installation

You can install the released version of wjake from
[CRAN](https://cran.r-project.org/) with:

``` r
install.packages("wjake")
```

To install the development version from [GitHub](https://github.com/)
use:

``` r
# install.packages("remotes")
remotes::install_github("wjakethompson/wjake")
```

## Examples

Create branded plots with `theme_wjake()`:

``` r
library(ggplot2)

ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
  geom_point(aes(color = species), na.rm = TRUE) +
  labs(
    x = "Bill length (mm)",
    y = "Flipper length (mm)",
    title = "Seminal ggplot2 scatterplot example",
    subtitle = "A plot that is only useful for demonstration purposes",
    caption = "Brought to you by \U1F427",
    color = "Species"
  ) +
  theme_wjake()
```

<img src="man/figures/README-penguin-plot-1.png" alt="" width="100%" />

And tables with `gt_theme_wjake()`:

``` r
library(gt)

head(penguins) |>
  gt() |>
  gt_theme_wjake(bg_color = "white") |>
  fmt_number(year, decimals = 0, use_seps = FALSE)
```

<div id="vsryprahzd" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
<style>#vsryprahzd table {
  font-family: 'Source Sans Pro', system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji';
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}
&#10;#vsryprahzd thead, #vsryprahzd tbody, #vsryprahzd tfoot, #vsryprahzd tr, #vsryprahzd td, #vsryprahzd th {
  border-style: none;
}
&#10;#vsryprahzd p {
  margin: 0;
  padding: 0;
}
&#10;#vsryprahzd .gt_table {
  display: table;
  border-collapse: collapse;
  line-height: normal;
  margin-left: 0;
  margin-right: auto;
  color: #333333;
  font-size: 16px;
  font-weight: normal;
  font-style: normal;
  background-color: #FFFFFF;
  width: auto;
  border-top-style: solid;
  border-top-width: 1px;
  border-top-color: #A8A8A8;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 1px;
  border-bottom-color: #A8A8A8;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_caption {
  padding-top: 4px;
  padding-bottom: 4px;
}
&#10;#vsryprahzd .gt_title {
  color: #333333;
  font-size: 125%;
  font-weight: initial;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-color: #FFFFFF;
  border-bottom-width: 0;
}
&#10;#vsryprahzd .gt_subtitle {
  color: #333333;
  font-size: 85%;
  font-weight: initial;
  padding-top: 3px;
  padding-bottom: 5px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-color: #FFFFFF;
  border-top-width: 0;
}
&#10;#vsryprahzd .gt_heading {
  background-color: #FFFFFF;
  text-align: left;
  border-bottom-color: #FFFFFF;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_bottom_border {
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_col_headings {
  border-top-style: none;
  border-top-width: 1px;
  border-top-color: #000000;
  border-bottom-style: solid;
  border-bottom-width: 3px;
  border-bottom-color: #000000;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_col_heading {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 80%;
  font-weight: bolder;
  text-transform: uppercase;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 6px;
  padding-left: 5px;
  padding-right: 5px;
  overflow-x: hidden;
}
&#10;#vsryprahzd .gt_column_spanner_outer {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 80%;
  font-weight: bolder;
  text-transform: uppercase;
  padding-top: 0;
  padding-bottom: 0;
  padding-left: 4px;
  padding-right: 4px;
}
&#10;#vsryprahzd .gt_column_spanner_outer:first-child {
  padding-left: 0;
}
&#10;#vsryprahzd .gt_column_spanner_outer:last-child {
  padding-right: 0;
}
&#10;#vsryprahzd .gt_column_spanner {
  border-bottom-style: solid;
  border-bottom-width: 3px;
  border-bottom-color: #000000;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 5px;
  overflow-x: hidden;
  display: inline-block;
  width: 100%;
}
&#10;#vsryprahzd .gt_spanner_row {
  border-bottom-style: hidden;
}
&#10;#vsryprahzd .gt_group_heading {
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  text-align: left;
}
&#10;#vsryprahzd .gt_empty_group_heading {
  padding: 0.5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  vertical-align: middle;
}
&#10;#vsryprahzd .gt_from_md > :first-child {
  margin-top: 0;
}
&#10;#vsryprahzd .gt_from_md > :last-child {
  margin-bottom: 0;
}
&#10;#vsryprahzd .gt_row {
  padding-top: 3px;
  padding-bottom: 3px;
  padding-left: 5px;
  padding-right: 5px;
  margin: 10px;
  border-top-style: solid;
  border-top-width: 1px;
  border-top-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  overflow-x: hidden;
}
&#10;#vsryprahzd .gt_stub {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#vsryprahzd .gt_stub_row_group {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
  vertical-align: top;
}
&#10;#vsryprahzd .gt_row_group_first td {
  border-top-width: 2px;
}
&#10;#vsryprahzd .gt_row_group_first th {
  border-top-width: 2px;
}
&#10;#vsryprahzd .gt_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#vsryprahzd .gt_first_summary_row {
  border-top-style: solid;
  border-top-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_first_summary_row.thick {
  border-top-width: 2px;
}
&#10;#vsryprahzd .gt_last_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_grand_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#vsryprahzd .gt_first_grand_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-style: double;
  border-top-width: 6px;
  border-top-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_last_grand_summary_row_top {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: double;
  border-bottom-width: 6px;
  border-bottom-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_striped {
  background-color: rgba(128, 128, 128, 0.05);
}
&#10;#vsryprahzd .gt_table_body {
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_footnotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_footnote {
  margin: 0px;
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#vsryprahzd .gt_sourcenotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 0px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 0px;
  border-right-color: #D3D3D3;
}
&#10;#vsryprahzd .gt_sourcenote {
  font-size: 12.8px;
  padding-top: 10px;
  padding-bottom: 10px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#vsryprahzd .gt_left {
  text-align: left;
}
&#10;#vsryprahzd .gt_center {
  text-align: center;
}
&#10;#vsryprahzd .gt_right {
  text-align: right;
  font-variant-numeric: tabular-nums;
}
&#10;#vsryprahzd .gt_font_normal {
  font-weight: normal;
}
&#10;#vsryprahzd .gt_font_bold {
  font-weight: bold;
}
&#10;#vsryprahzd .gt_font_italic {
  font-style: italic;
}
&#10;#vsryprahzd .gt_super {
  font-size: 65%;
}
&#10;#vsryprahzd .gt_footnote_marks {
  font-size: 75%;
  vertical-align: 0.4em;
  position: initial;
}
&#10;#vsryprahzd .gt_asterisk {
  font-size: 100%;
  vertical-align: 0;
}
&#10;#vsryprahzd .gt_indent_1 {
  text-indent: 5px;
}
&#10;#vsryprahzd .gt_indent_2 {
  text-indent: 10px;
}
&#10;#vsryprahzd .gt_indent_3 {
  text-indent: 15px;
}
&#10;#vsryprahzd .gt_indent_4 {
  text-indent: 20px;
}
&#10;#vsryprahzd .gt_indent_5 {
  text-indent: 25px;
}
&#10;#vsryprahzd .katex-display {
  display: inline-flex !important;
  margin-bottom: 0.75em !important;
}
&#10;#vsryprahzd div.Reactable > div.rt-table > div.rt-thead > div.rt-tr.rt-tr-group-header > div.rt-th-group:after {
  height: 0px !important;
}
</style>
<table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <thead>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="species">species</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="island">island</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="bill_len">bill_len</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="bill_dep">bill_dep</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="flipper_len">flipper_len</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="body_mass">body_mass</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="sex">sex</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="year">year</th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.1</td>
<td headers="bill_dep" class="gt_row gt_right">18.7</td>
<td headers="flipper_len" class="gt_row gt_right">181 </td>
<td headers="body_mass" class="gt_row gt_right">3,750 </td>
<td headers="sex" class="gt_row gt_center">male</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.5</td>
<td headers="bill_dep" class="gt_row gt_right">17.4</td>
<td headers="flipper_len" class="gt_row gt_right">186 </td>
<td headers="body_mass" class="gt_row gt_right">3,800 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">40.3</td>
<td headers="bill_dep" class="gt_row gt_right">18.0</td>
<td headers="flipper_len" class="gt_row gt_right">195 </td>
<td headers="body_mass" class="gt_row gt_right">3,250 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">NA</td>
<td headers="bill_dep" class="gt_row gt_right">NA</td>
<td headers="flipper_len" class="gt_row gt_right">NA</td>
<td headers="body_mass" class="gt_row gt_right">NA</td>
<td headers="sex" class="gt_row gt_center">NA</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">36.7</td>
<td headers="bill_dep" class="gt_row gt_right">19.3</td>
<td headers="flipper_len" class="gt_row gt_right">193 </td>
<td headers="body_mass" class="gt_row gt_right">3,450 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.3</td>
<td headers="bill_dep" class="gt_row gt_right">20.6</td>
<td headers="flipper_len" class="gt_row gt_right">190 </td>
<td headers="body_mass" class="gt_row gt_right">3,650 </td>
<td headers="sex" class="gt_row gt_center">male</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
  </tbody>
  <tfoot>
    <tr class="gt_sourcenotes">
      <td class="gt_sourcenote" style="text-align: right; text-transform: uppercase;" colspan="8">Table: @wjakethompson.com</td>
    </tr>
  </tfoot>
</table>
</div>

## Code of Conduct

Please note that the wjake project is released with a [Contributor Code
of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# wjake <a href="https://wjake.wjakethompson.com"><img src="man/figures/logo.png" align="right" height="139" alt="wjake website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![lifecycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![R package
version](https://www.r-pkg.org/badges/version/wjake)](https://CRAN.R-project.org/package=wjake)
[![Package
downloads](https://cranlogs.r-pkg.org/badges/grand-total/wjake)](https://cran.r-project.org/package=wjake)</br>
[![R-CMD-check](https://github.com/wjakethompson/wjake/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/wjakethompson/wjake/actions/workflows/R-CMD-check.yaml)
[![codecov](https://codecov.io/gh/wjakethompson/wjake/branch/main/graph/badge.svg?token=i9e5Zt18sa)](https://app.codecov.io/gh/wjakethompson/wjake)
[![pages-build-deployment](https://github.com/wjakethompson/wjake/actions/workflows/pages/pages-build-deployment/badge.svg)](https://github.com/wjakethompson/wjake/actions/workflows/pages/pages-build-deployment)</br>
[![Signed
by](https://img.shields.io/badge/Keybase-Verified-brightgreen.svg)](https://keybase.io/wjakethompson)
![License](https://img.shields.io/badge/License-MIT-blue.svg)
<!-- badges: end -->

This is a personal R package containing utility functions and custom
themes for [ggplot2](https://ggplot2.tidyverse.org/) and
[gt](https://gt.rstudio.com/).

## Installation

You can install the released version of wjake from
[CRAN](https://cran.r-project.org/) with:

``` r
install.packages("wjake")
```

To install the development version from [GitHub](https://github.com/)
use:

``` r
# install.packages("remotes")
remotes::install_github("wjakethompson/wjake")
```

## Examples

Create branded plots with `theme_wjake()`:

``` r
library(ggplot2)

ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
  geom_point(aes(color = species), na.rm = TRUE) +
  labs(
    x = "Bill length (mm)",
    y = "Flipper length (mm)",
    title = "Seminal ggplot2 scatterplot example",
    subtitle = "A plot that is only useful for demonstration purposes",
    caption = "Brought to you by \U1F427",
    color = "Species"
  ) +
  theme_wjake()
```

<img src="man/figures/README-penguin-plot-1.png" alt="" width="100%" />

And tables with `gt_theme_wjake()`:

``` r
library(gt)

head(penguins) |>
  gt() |>
  gt_theme_wjake(bg_color = "white") |>
  fmt_number(year, decimals = 0, use_seps = FALSE)
```

<div id="obazwdgjnc" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
<style>#obazwdgjnc table {
  font-family: 'Source Sans Pro', system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji';
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}
&#10;#obazwdgjnc thead, #obazwdgjnc tbody, #obazwdgjnc tfoot, #obazwdgjnc tr, #obazwdgjnc td, #obazwdgjnc th {
  border-style: none;
}
&#10;#obazwdgjnc p {
  margin: 0;
  padding: 0;
}
&#10;#obazwdgjnc .gt_table {
  display: table;
  border-collapse: collapse;
  line-height: normal;
  margin-left: 0;
  margin-right: auto;
  color: #333333;
  font-size: 16px;
  font-weight: normal;
  font-style: normal;
  background-color: #FFFFFF;
  width: auto;
  border-top-style: solid;
  border-top-width: 1px;
  border-top-color: #A8A8A8;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 1px;
  border-bottom-color: #A8A8A8;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_caption {
  padding-top: 4px;
  padding-bottom: 4px;
}
&#10;#obazwdgjnc .gt_title {
  color: #333333;
  font-size: 125%;
  font-weight: initial;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-color: #FFFFFF;
  border-bottom-width: 0;
}
&#10;#obazwdgjnc .gt_subtitle {
  color: #333333;
  font-size: 85%;
  font-weight: initial;
  padding-top: 3px;
  padding-bottom: 5px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-color: #FFFFFF;
  border-top-width: 0;
}
&#10;#obazwdgjnc .gt_heading {
  background-color: #FFFFFF;
  text-align: left;
  border-bottom-color: #FFFFFF;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_bottom_border {
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_col_headings {
  border-top-style: none;
  border-top-width: 1px;
  border-top-color: #000000;
  border-bottom-style: solid;
  border-bottom-width: 3px;
  border-bottom-color: #000000;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_col_heading {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 80%;
  font-weight: bolder;
  text-transform: uppercase;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 6px;
  padding-left: 5px;
  padding-right: 5px;
  overflow-x: hidden;
}
&#10;#obazwdgjnc .gt_column_spanner_outer {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 80%;
  font-weight: bolder;
  text-transform: uppercase;
  padding-top: 0;
  padding-bottom: 0;
  padding-left: 4px;
  padding-right: 4px;
}
&#10;#obazwdgjnc .gt_column_spanner_outer:first-child {
  padding-left: 0;
}
&#10;#obazwdgjnc .gt_column_spanner_outer:last-child {
  padding-right: 0;
}
&#10;#obazwdgjnc .gt_column_spanner {
  border-bottom-style: solid;
  border-bottom-width: 3px;
  border-bottom-color: #000000;
  vertical-align: bottom;
  padding-top: 5px;
  padding-bottom: 5px;
  overflow-x: hidden;
  display: inline-block;
  width: 100%;
}
&#10;#obazwdgjnc .gt_spanner_row {
  border-bottom-style: hidden;
}
&#10;#obazwdgjnc .gt_group_heading {
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  text-align: left;
}
&#10;#obazwdgjnc .gt_empty_group_heading {
  padding: 0.5px;
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  vertical-align: middle;
}
&#10;#obazwdgjnc .gt_from_md > :first-child {
  margin-top: 0;
}
&#10;#obazwdgjnc .gt_from_md > :last-child {
  margin-bottom: 0;
}
&#10;#obazwdgjnc .gt_row {
  padding-top: 3px;
  padding-bottom: 3px;
  padding-left: 5px;
  padding-right: 5px;
  margin: 10px;
  border-top-style: solid;
  border-top-width: 1px;
  border-top-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 1px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 1px;
  border-right-color: #D3D3D3;
  vertical-align: middle;
  overflow-x: hidden;
}
&#10;#obazwdgjnc .gt_stub {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#obazwdgjnc .gt_stub_row_group {
  color: #333333;
  background-color: #FFFFFF;
  font-size: 100%;
  font-weight: initial;
  text-transform: inherit;
  border-right-style: solid;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
  padding-left: 5px;
  padding-right: 5px;
  vertical-align: top;
}
&#10;#obazwdgjnc .gt_row_group_first td {
  border-top-width: 2px;
}
&#10;#obazwdgjnc .gt_row_group_first th {
  border-top-width: 2px;
}
&#10;#obazwdgjnc .gt_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#obazwdgjnc .gt_first_summary_row {
  border-top-style: solid;
  border-top-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_first_summary_row.thick {
  border-top-width: 2px;
}
&#10;#obazwdgjnc .gt_last_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_grand_summary_row {
  color: #333333;
  background-color: #FFFFFF;
  text-transform: inherit;
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#obazwdgjnc .gt_first_grand_summary_row {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-top-style: double;
  border-top-width: 6px;
  border-top-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_last_grand_summary_row_top {
  padding-top: 8px;
  padding-bottom: 8px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom-style: double;
  border-bottom-width: 6px;
  border-bottom-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_striped {
  background-color: rgba(128, 128, 128, 0.05);
}
&#10;#obazwdgjnc .gt_table_body {
  border-top-style: none;
  border-top-width: 2px;
  border-top-color: #D3D3D3;
  border-bottom-style: solid;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_footnotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 2px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 2px;
  border-right-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_footnote {
  margin: 0px;
  font-size: 90%;
  padding-top: 4px;
  padding-bottom: 4px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#obazwdgjnc .gt_sourcenotes {
  color: #333333;
  background-color: #FFFFFF;
  border-bottom-style: none;
  border-bottom-width: 2px;
  border-bottom-color: #D3D3D3;
  border-left-style: none;
  border-left-width: 0px;
  border-left-color: #D3D3D3;
  border-right-style: none;
  border-right-width: 0px;
  border-right-color: #D3D3D3;
}
&#10;#obazwdgjnc .gt_sourcenote {
  font-size: 12.8px;
  padding-top: 10px;
  padding-bottom: 10px;
  padding-left: 5px;
  padding-right: 5px;
}
&#10;#obazwdgjnc .gt_left {
  text-align: left;
}
&#10;#obazwdgjnc .gt_center {
  text-align: center;
}
&#10;#obazwdgjnc .gt_right {
  text-align: right;
  font-variant-numeric: tabular-nums;
}
&#10;#obazwdgjnc .gt_font_normal {
  font-weight: normal;
}
&#10;#obazwdgjnc .gt_font_bold {
  font-weight: bold;
}
&#10;#obazwdgjnc .gt_font_italic {
  font-style: italic;
}
&#10;#obazwdgjnc .gt_super {
  font-size: 65%;
}
&#10;#obazwdgjnc .gt_footnote_marks {
  font-size: 75%;
  vertical-align: 0.4em;
  position: initial;
}
&#10;#obazwdgjnc .gt_asterisk {
  font-size: 100%;
  vertical-align: 0;
}
&#10;#obazwdgjnc .gt_indent_1 {
  text-indent: 5px;
}
&#10;#obazwdgjnc .gt_indent_2 {
  text-indent: 10px;
}
&#10;#obazwdgjnc .gt_indent_3 {
  text-indent: 15px;
}
&#10;#obazwdgjnc .gt_indent_4 {
  text-indent: 20px;
}
&#10;#obazwdgjnc .gt_indent_5 {
  text-indent: 25px;
}
&#10;#obazwdgjnc .katex-display {
  display: inline-flex !important;
  margin-bottom: 0.75em !important;
}
&#10;#obazwdgjnc div.Reactable > div.rt-table > div.rt-thead > div.rt-tr.rt-tr-group-header > div.rt-th-group:after {
  height: 0px !important;
}
</style>
<table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <thead>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="species">species</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="island">island</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="bill_len">bill_len</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="bill_dep">bill_dep</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="flipper_len">flipper_len</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="body_mass">body_mass</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="sex">sex</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" style="text-align: center; vertical-align: middle;" scope="col" id="year">year</th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.1</td>
<td headers="bill_dep" class="gt_row gt_right">18.7</td>
<td headers="flipper_len" class="gt_row gt_right">181 </td>
<td headers="body_mass" class="gt_row gt_right">3,750 </td>
<td headers="sex" class="gt_row gt_center">male</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.5</td>
<td headers="bill_dep" class="gt_row gt_right">17.4</td>
<td headers="flipper_len" class="gt_row gt_right">186 </td>
<td headers="body_mass" class="gt_row gt_right">3,800 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">40.3</td>
<td headers="bill_dep" class="gt_row gt_right">18.0</td>
<td headers="flipper_len" class="gt_row gt_right">195 </td>
<td headers="body_mass" class="gt_row gt_right">3,250 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">NA</td>
<td headers="bill_dep" class="gt_row gt_right">NA</td>
<td headers="flipper_len" class="gt_row gt_right">NA</td>
<td headers="body_mass" class="gt_row gt_right">NA</td>
<td headers="sex" class="gt_row gt_center">NA</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">36.7</td>
<td headers="bill_dep" class="gt_row gt_right">19.3</td>
<td headers="flipper_len" class="gt_row gt_right">193 </td>
<td headers="body_mass" class="gt_row gt_right">3,450 </td>
<td headers="sex" class="gt_row gt_center">female</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
    <tr><td headers="species" class="gt_row gt_center">Adelie</td>
<td headers="island" class="gt_row gt_center">Torgersen</td>
<td headers="bill_len" class="gt_row gt_right">39.3</td>
<td headers="bill_dep" class="gt_row gt_right">20.6</td>
<td headers="flipper_len" class="gt_row gt_right">190 </td>
<td headers="body_mass" class="gt_row gt_right">3,650 </td>
<td headers="sex" class="gt_row gt_center">male</td>
<td headers="year" class="gt_row gt_right">2007 </td></tr>
  </tbody>
  <tfoot>
    <tr class="gt_sourcenotes">
      <td class="gt_sourcenote" style="text-align: right; text-transform: uppercase;" colspan="8">Table: @wjakethompson.com</td>
    </tr>
  </tfoot>
</table>
</div>

## Code of Conduct

Please note that the wjake project is released with a [Contributor Code
of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' wjake: A package for package installation and miscellaneous functions
#'
#' Personal package for helping with R setup and random tasks that I Google too
#' often.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import ggplot2
#' @import rlang
## usethis namespace: end
NULL

## Make R CMD Check go away
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c("."))
}

#' Okabe-Ito color palette
#'
#' Colorblind friendly palette taken originally described by Okabe and Ito
#' (2008). The original palette has been modified slightly to make the yellow
#' more visible on a white background, as described by Lüdecke et al. (2021).
#' The palette also includes color scales that can be used in place of
#' [ggplot2::scale_color_discrete()] or [ggplot2::scale_fill_discrete()].
#'
#' @references Lüdecke, D., Patil, I., Ben-Shachar, M. S., Wiernik, B. M.,
#'   Waggoner, P., & Makowski, D. (2021). see: An R package for visualizing
#'   statistical models. *Journal of Open Source Software, 6*(64), Article 3393.
#'   \doi{10.21105/joss.03393}
#' @references Okabe, M., & Ito, K. (2008). Color universal design (CUD): How to
#'   make figures and presentations that are friendly to colorblind people.
#'   <https://jfly.uni-koeln.de/color/#pallet> (Original work published 2002)
#'
#' @format A character vector of 8 hex color codes.
#' @export
#' @examples
#' scales::show_col(palette_okabeito)
palette_okabeito <- c(
  "#E69F00",
  "#56B4E9",
  "#009E73",
  "#F5C710",
  "#0072B2",
  "#D55E00",
  "#CC79A7",
  "#000000"
)

#' @rdname palette_okabeito
#' @export
#' @usage NULL
scale_colour_okabeito <- function(aesthetics = "colour", ...) {
  scale_palette(aesthetics, palette = palette_okabeito, ...)
}

#' @rdname palette_okabeito
#' @export
#' @usage NULL
scale_color_okabeito <- scale_colour_okabeito

#' @rdname palette_okabeito
#' @export
#' @usage NULL
scale_fill_okabeito <- function(aesthetics = "fill", ...) {
  scale_palette(aesthetics, palette = palette_okabeito, ...)
}

test_that("okabe ito palette is correct", {
  expect_equal(
    palette_okabeito,
    c(
      "#E69F00",
      "#56B4E9",
      "#009E73",
      "#F5C710",
      "#0072B2",
      "#D55E00",
      "#CC79A7",
      "#000000"
    )
  )
})

test_that("okabe ito scales work", {
  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    theme_bw()
  vdiffr::expect_doppelganger("okabeito-color", p + scale_color_okabeito())
  vdiffr::expect_doppelganger("okabeito-colour", p + scale_colour_okabeito())

  p <- ggplot(penguins, aes(x = bill_len)) +
    geom_density(aes(fill = species), na.rm = TRUE, alpha = 0.6) +
    scale_fill_okabeito()
  vdiffr::expect_doppelganger("okabeito-fill", p)
})

#' Title
#'
#' @param data The gt table data object.
#' @param dec_dig The number of decimal places to round to for numeric values.
#' @param corr_dig The number of decimal places to round to for numeric values
#'   all constrained to be between \[-1, 1\].
#' @param fmt_extreme Logical indicator for whether values close to extremes
#'   should be suppressed (e.g., .000001 becomes <.001).
#' @param bg_color The background color of the table.
#' @param font The font to use for the table.
#' @param font_size The base font size for the table.
#' @param ... Additional options passed to [gt::tab_options()]
#'
#' @returns An object of class `gt-tbl`.
#'
#' @export
#' @examples
#' gt::gt(head(penguins)) |>
#'   gt_theme_apa() |>
#'   gt::fmt_number(year, sep_mark = "", decimals = 0)
gt_theme_apa <- function(
  data,
  dec_dig = 1,
  corr_dig = 3,
  fmt_extreme = TRUE,
  bg_color = "#FFFFFF",
  font = "Arial",
  font_size = 11,
  ...
) {
  # general formatting -----
  data <- data |>
    gt::cols_align_decimal(columns = dplyr::where(is.numeric)) |>
    gt::opt_table_font(font = list(font))

  # number formatting -----
  data <- data |>
    ## integers
    gt::fmt_number(columns = dplyr::where(is.integer), decimals = 0) |>

    ## numerics
    gt::fmt_number(
      columns = dplyr::where(\(x) {
        is.double(x) && !all(dplyr::between(x, -1, 1))
      }),
      decimals = dec_dig
    ) |>

    ## proportions and correlations bounded [-1, 1]
    gt::fmt_number(
      columns = dplyr::where(\(x) {
        is.double(x) && all(dplyr::between(x, -1, 1))
      }),
      decimals = 3
    )

  # extreme value formatting -----
  if (fmt_extreme) {
    data <- data |>
      gt::sub_small_vals(
        columns = dplyr::where(\(x) {
          is.double(x) && !all(dplyr::between(x, -1, 1))
        }),
        threshold = 1 / (10^dec_dig)
      ) |>
      gt::sub_large_vals(
        columns = dplyr::where(\(x) {
          is.double(x) && !all(dplyr::between(x, -1, 1))
        }),
        threshold = 100 - (1 / (10^dec_dig)),
        large_pattern = ">{x}"
      ) |>
      gt::sub_large_vals(
        columns = dplyr::where(\(x) {
          is.double(x) && all(dplyr::between(x, -1, 1))
        }),
        threshold = 1 - (1 / (10^corr_dig)),
        large_pattern = ">{x}"
      ) |>
      gt::sub_large_vals(
        columns = dplyr::where(\(x) {
          is.double(x) && all(dplyr::between(x, -1, 1))
        }),
        threshold = 1 - (1 / (10^corr_dig)),
        large_pattern = "<\U2212{x}",
        sign = "-"
      )
  }

  # group formatting -----
  if (length(data$`_row_groups`)) {
    data <- data |>
      gt::tab_style(
        style = gt::cell_text(indent = gt::px(20)),
        locations = gt::cells_body(
          columns = 1,
          rows = gt::everything()
        )
      )
  }

  # table styling -----
  data <- data |>
    gt::text_transform(
      fn = \(x) gsub("0\\.", ".", x),
      locations = gt::cells_body(
        columns = dplyr::where(\(x) {
          is.double(x) && all(dplyr::between(x, -1, 1))
        }),
        rows = gt::everything()
      )
    ) |>
    gt::tab_options(
      table.align = "left",
      table.background.color = bg_color
    ) |>
    gt::tab_options(
      table.font.size = font_size,
      source_notes.font.size = font_size * .8,
      data_row.padding = gt::px(3),
      row_group.padding = gt::px(4),
      table_body.hlines.style = "none",
      row_group.border.top.style = "none",
      row_group.border.bottom.style = "none",
      table.border.top.style = "none",
      table.border.bottom.style = "none",
      column_labels.border.top.style = "solid",
      column_labels.border.top.width = gt::px(1),
      column_labels.border.top.color = "black",
      column_labels.border.bottom.style = "solid",
      column_labels.border.bottom.width = gt::px(1),
      column_labels.border.bottom.color = "black",
      table_body.border.top.style = "none",
      table_body.border.bottom.style = "solid",
      table_body.border.bottom.width = gt::px(1),
      table_body.border.bottom.color = "black",
      ...
    ) |>
    gt::tab_style(
      style = list(
        gt::cell_text(align = "center", v_align = "middle")
      ),
      locations = gt::cells_column_labels(gt::everything())
    )

  data
}

test_that("gt_theme_wjake() returns a gt_tbl", {
  tbl <- gt::gt(data.frame(x = 1.0)) |> gt_theme_wjake()
  expect_s3_class(tbl, "gt_tbl")
})

test_that("gt_theme_wjake() adds the source note", {
  tbl <- gt::gt(data.frame(x = 1.0)) |> gt_theme_wjake()
  expect_equal(tbl[["_source_notes"]][[1]], "Table: @wjakethompson.com")
})

test_that("gt_theme_wjake() inherits gt_theme_apa() number formatting", {
  df <- data.frame(
    int_col = c(1L, 2L),
    dbl_col = c(1.5, 20.0),
    corr_col = c(0.123, 0.456)
  )
  tbl <- gt::gt(df) |> gt_theme_wjake()
  expect_snapshot(gt::extract_body(tbl))
})

test_that("gt_theme_apa() returns a gt_tbl", {
  tbl <- gt::gt(data.frame(x = 1.0)) |> gt_theme_apa()
  expect_s3_class(tbl, "gt_tbl")
})

test_that("gt_theme_apa() formats integers with 0 decimals", {
  tbl <- gt::gt(data.frame(x = c(1L, 10L, 100L))) |> gt_theme_apa()
  expect_snapshot(gt::extract_body(tbl))
})

test_that("gt_theme_apa() formats doubles with dec_dig decimals", {
  tbl1 <- gt::gt(data.frame(x = c(1.234, 56.789))) |> gt_theme_apa(dec_dig = 1)
  tbl2 <- gt::gt(data.frame(x = c(1.234, 56.789))) |> gt_theme_apa(dec_dig = 2)
  expect_snapshot(gt::extract_body(tbl1))
  expect_snapshot(gt::extract_body(tbl2))
})

test_that("gt_theme_apa() has correlations with 3 digits and no leading zero", {
  tbl <- gt::gt(data.frame(r = c(-0.5, 0.0, 0.123, 0.456))) |> gt_theme_apa()
  expect_snapshot(gt::extract_body(tbl))
})

test_that("gt_theme_apa() substitutes extreme doubles based on dec_dig", {
  df <- data.frame(x = c(0.05, 0.1, 99.9, 99.95))
  tbl1 <- gt::gt(df) |> gt_theme_apa(dec_dig = 1)
  tbl2 <- gt::gt(df) |> gt_theme_apa(dec_dig = 2)
  expect_snapshot(gt::extract_body(tbl1))
  expect_snapshot(gt::extract_body(tbl2))
})

test_that("gt_theme_apa() substitutes extreme correlations based on corr_dig", {
  df <- data.frame(r = c(-0.9999, -0.999, 0.999, 0.9999))
  tbl1 <- gt::gt(df) |> gt_theme_apa(corr_dig = 3)
  tbl2 <- gt::gt(df) |> gt_theme_apa(corr_dig = 2)
  expect_snapshot(gt::extract_body(tbl1))
  expect_snapshot(gt::extract_body(tbl2))
})

test_that("gt_theme_apa() can skip extreme substitution", {
  df <- data.frame(x = c(0.06, 99.96), r = c(-0.9999, 0.9999))
  tbl <- gt::gt(df) |> gt_theme_apa(dec_dig = 1, fmt_extreme = FALSE)
  expect_snapshot(gt::extract_body(tbl))
})

test_that("gt_theme_apa() returns a gt_tbl for grouped input", {
  df <- data.frame(group = c("A", "A", "B"), x = c(1.1, 2.2, 3.3))
  tbl <- gt::gt(df, groupname_col = "group") |> gt_theme_apa()
  expect_s3_class(tbl, "gt_tbl")
  expect_equal(tbl[["_row_groups"]], c("A", "B"))
})

test_that("gt_theme_apa() applies number formatting in grouped tables", {
  df <- data.frame(
    group = c("A", "A", "B"),
    int_col = c(1L, 2L, 3L),
    dbl_col = c(1.5, 20.0, 99.95),
    corr_col = c(0.123, 0.456, 0.789)
  )
  tbl <- gt::gt(df, groupname_col = "group") |> gt_theme_apa(dec_dig = 1)
  expect_snapshot(gt::extract_body(tbl))
})

# nocov start
.onAttach <- function(libname, pkgname) {
  if (getOption("wjake.loadfonts", default = TRUE)) {
    if (!("Source Sans Pro" %in% showtextdb::font_installed())) {
      showtextdb::font_install(showtextdb::google_fonts("Source Sans Pro"))
    }

    font_files <-
      system.file(
        "fonts",
        "Source Sans Pro",
        c("regular.ttf", "bold.ttf", "italic.ttf", "bolditalic.ttf"),
        package = "showtextdb"
      )

    sysfonts::font_add(
      family = "Source Sans Pro",
      regular = font_files[1],
      bold = font_files[2],
      italic = font_files[3],
      bolditalic = font_files[4]
    )

    if (!"Source Sans Pro" %in% systemfonts::system_fonts()$family) {
      systemfonts::register_font(
        name = "Source Sans Pro",
        plain = font_files[1],
        bold = font_files[2],
        italic = font_files[3],
        bolditalic = font_files[4]
      )
    }
  }
}
# nocov end

#' Round to a specified value
#'
#' @param x A numeric value to round.
#' @param accuracy The accuracy with which to round (i.e., round to the nearest
#'   `accuracy`).
#' @param direction The direction to round. `nearest` (default) will round to
#'   the nearest `accuracy`, `up` will round up to the nearest `accuracy`,
#'   `down` will round down to the nearest `accuracy`, and `random` will
#'   randomly round up or down to the nearest `accuracy` as described by
#'   Matthews & Harel (2011).
#'
#' @return A numeric rounded to the specified accuracy.
#' @export
#'
#' @references Matthews, G., & Harel, O. (2011). Data confidentiality: A review
#'   of methods for statistical disclosure limitation and methods for assessing
#'   privacy. *Statistics Surveys, 5*, 1--29. \doi{10.1214/11-SS074}
#'
#' @examples
#' round_to(15, accuracy = 7, direction = "nearest")
#' round_to(15, accuracy = 7, direction = "up")
#' round_to(20, accuracy = 7, direction = "down")
round_to <- function(
  x,
  accuracy,
  direction = c("nearest", "up", "down", "random")
) {
  direction <- rlang::arg_match(direction)

  switch(
    direction,
    nearest = round_nr(x, accuracy),
    up = round_up(x, accuracy),
    down = round_dn(x, accuracy),
    random = round_rand(x, accuracy)
  )
}

round_nr <- function(x, accuracy) {
  round(x / accuracy) * accuracy
}

round_dn <- function(x, accuracy) {
  floor(x / accuracy) * accuracy
}

round_up <- function(x, accuracy) {
  ceiling(x / accuracy) * accuracy
}

round_rand <- function(x, accuracy) {
  if (accuracy == 0) {
    return(x)
  }

  dn <- round_dn(x, accuracy)
  up <- round_up(x, accuracy)

  probs <- 1 - c((x - dn) / accuracy, (up - x) / accuracy)

  sample(c(dn, up), prob = probs, size = 1, replace = FALSE)
}

test_that("rounding works", {
  expect_equal(round_to(15, accuracy = 7, direction = "nearest"), 14)
  expect_equal(round_to(51, accuracy = 10, direction = "nearest"), 50)
  expect_equal(round_to(25, accuracy = 1, direction = "nearest"), 25)
  expect_equal(round_to(22, accuracy = 8, direction = "nearest"), 24)
  expect_equal(round_to(58, accuracy = 2, direction = "nearest"), 58)

  expect_equal(round_to(66, accuracy = 6, direction = "up"), 66)
  expect_equal(round_to(39, accuracy = 9, direction = "up"), 45)
  expect_equal(round_to(65, accuracy = 1, direction = "up"), 65)
  expect_equal(round_to(45, accuracy = 3, direction = "up"), 45)
  expect_equal(round_to(79, accuracy = 5, direction = "up"), 80)

  expect_equal(round_to(81, accuracy = 3, direction = "down"), 81)
  expect_equal(round_to(59, accuracy = 6, direction = "down"), 54)
  expect_equal(round_to(31, accuracy = 5, direction = "down"), 30)
  expect_equal(round_to(5, accuracy = 1, direction = "down"), 5)
  expect_equal(round_to(69, accuracy = 7, direction = "down"), 63)

  expect_equal(round_to(13, accuracy = 0, direction = "random"), 13)

  rand1 <- vapply(
    rep(17L, 1e5),
    round_to,
    double(1),
    accuracy = 10,
    direction = "random"
  )
  expect_equal(mean(rand1 == 20), .7, tolerance = 0.1)

  rand2 <- vapply(
    rep(68L, 1e5),
    round_to,
    double(1),
    accuracy = 3,
    direction = "random"
  )
  expect_equal(mean(rand2 == 66), .33, tolerance = 0.1)
})

#' Personal color palette
#'
#' Color palette used for theming https://wjakethompson.com. The palette also
#' includes color scales that can be used in place of
#' [ggplot2::scale_color_discrete()] or [ggplot2::scale_fill_discrete()].
#'
#' @format A character vector of 5 hex color codes.
#' @export
#' @examples
#' scales::show_col(palette_wjake)
palette_wjake <- c(
  "#009FB7",
  "#FED766",
  "#272727",
  "#696773",
  "#F0F0F0"
)

#' @rdname palette_wjake
#' @export
#' @usage NULL
scale_colour_wjake <- function(aesthetics = "colour", ...) {
  scale_palette(aesthetics, palette = palette_wjake, ...)
}

#' @rdname palette_wjake
#' @export
#' @usage NULL
scale_color_wjake <- scale_colour_wjake

#' @rdname palette_wjake
#' @export
#' @usage NULL
scale_fill_wjake <- function(aesthetics = "fill", ...) {
  scale_palette(aesthetics, palette = palette_wjake, ...)
}

test_that("wjake palette is correct", {
  expect_equal(
    palette_wjake,
    c(
      "#009FB7",
      "#FED766",
      "#272727",
      "#696773",
      "#F0F0F0"
    )
  )
})

test_that("okabe ito scales work", {
  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    theme_bw()
  vdiffr::expect_doppelganger("wjake-color", p + scale_color_wjake())
  vdiffr::expect_doppelganger("wjake-colour", p + scale_colour_wjake())

  p <- ggplot(penguins, aes(x = bill_len)) +
    geom_density(aes(fill = species), na.rm = TRUE, alpha = 0.6) +
    scale_fill_wjake()
  vdiffr::expect_doppelganger("wjake-fill", p)
})

#' Personalized ggplot2 theme
#'
#' Based on [ggplot2::theme_minimal()].
#'
#' @inheritParams ggplot2::theme_minimal
#' @param continuous A character vector of valid colors that will be
#'   interpolated into a continuous color scale.
#' @param discrete A character vector of colors to use for discrete color
#'   scales.
#' @param transparent Logical indicator for whether the background of the plot
#'   should be transparent.
#' @param ... Additional parameters passed to [ggplot2::theme()].
#'
#' @return A `theme` object that can be added to a [ggplot2::ggplot()].
#' @export
#'
#' @examples
#' library(ggplot2)
#'
#' ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
#'   geom_point(aes(color = species), na.rm = TRUE) +
#'   labs(
#'     x = "Bill length (mm)",
#'     y = "Flipper length (mm)",
#'     title = "Seminal ggplot2 scatterplot example",
#'     subtitle = "A plot that is only useful for demonstration purposes",
#'     caption = "Brought to you by the letter *p*",
#'     color = "Species"
#'   ) +
#'   theme_wjake(base_family = "sans")
theme_wjake <- function(
  base_size = 11.5,
  base_family = "Source Sans Pro",
  header_family = NULL,
  base_line_size = base_size / 22,
  base_rect_size = base_size / 22,
  ink = "black",
  paper = "white",
  accent = "#FED766",
  continuous = ramp_blue(c(0.1, 1), end = "#FFFFFF"),
  discrete = palette_wjake,
  transparent = FALSE,
  ...
) {
  wjt_theme <- ggplot2::theme_minimal(
    base_size = base_size,
    base_family = base_family,
    header_family = header_family,
    base_line_size = base_line_size,
    base_rect_size = base_rect_size,
    ink = ink,
    paper = paper,
    accent = accent
  ) +
    ggplot2::theme_sub_legend(
      background = ggplot2::element_blank(),
      key = element_blank(),
      position = "bottom"
    ) +
    ggplot2::theme_sub_axis_bottom(
      title = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(.8),
        hjust = 1
      ),
      text = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = base_size
      )
    ) +
    ggplot2::theme_sub_axis_top(
      title = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(.8),
        hjust = 1
      ),
      text = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = base_size
      )
    ) +
    ggplot2::theme_sub_axis_left(
      title = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(.8),
        hjust = 1
      ),
      text = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = base_size
      )
    ) +
    ggplot2::theme_sub_axis_right(
      title = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(.8),
        hjust = 1
      ),
      text = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = base_size
      )
    ) +
    ggplot2::theme_sub_strip(
      text = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = base_size
      )
    ) +
    ggplot2::theme_sub_panel(
      spacing = unit(1, "lines")
    ) +
    ggplot2::theme_sub_plot(
      title = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(1.5)
      ),
      title.position = "plot",
      subtitle = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(1.2)
      ),
      caption = ggtext::element_markdown(
        family = base_family,
        color = "black",
        size = rel(.8)
      )
    ) +
    ggplot2::theme(
      geom = ggplot2::element_geom(
        ink = palette_wjake[1]
      ),
      palette.color.continuous = continuous,
      palette.fill.continuous = continuous,
      palette.color.discrete = discrete,
      palette.fill.discrete = discrete,
      ...
    )

  if (transparent) {
    wjt_theme <- wjt_theme +
      ggplot2::theme(
        plot.background = ggplot2::element_rect(
          fill = "transparent",
          color = "transparent"
        ),
        panel.background = ggplot2::element_rect(
          fill = "transparent",
          color = "transparent"
        ),
        legend.background = ggplot2::element_rect(
          fill = "transparent",
          color = "transparent"
        ),
        panel.border = ggplot2::element_blank()
      )
  }

  wjt_theme
}

test_that("theme_wjake() fonts and axes", {
  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(na.rm = TRUE) +
    labs(
      x = "**Bill length** (mm)",
      y = "*Flipper length* (mm)",
      title = "**Example plot**",
      subtitle = "Plot subtitle"
    ) +
    theme_wjake()
  vdiffr::expect_doppelganger("default-scatter", p)

  p <- ggplot(penguins, aes(x = bill_len)) +
    geom_histogram(na.rm = TRUE, bins = 30) +
    labs(
      x = "**Bill length** (mm)",
      y = "*n*",
      title = "**Example plot**",
      subtitle = "Plot subtitle"
    ) +
    theme_wjake(transparent = TRUE)
  vdiffr::expect_doppelganger("default-histogram", p)
})

test_that("theme_wjake() palettes", {
  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    labs(
      x = "**Bill length** (mm)",
      y = "*Flipper length* (mm)",
      title = "**Example plot**",
      subtitle = "Plot subtitle"
    ) +
    theme_wjake(transparent = TRUE)
  vdiffr::expect_doppelganger("discrete-color", p)

  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_bin_2d(na.rm = TRUE, bins = 30) +
    labs(
      x = "**Bill length** (mm)",
      y = "*Flipper length* (mm)",
      title = "**Example plot**",
      subtitle = "Plot subtitle"
    ) +
    theme_wjake()
  vdiffr::expect_doppelganger("continuous-color", p)
})

#' Write a bibliography for R packages
#'
#' @param pkg A vector of packages to create a `.bib` file for.
#' @param file The file to save the references.
#' @param update Should packages be updated before creating the bibliography?
#'
#' @return A list containing the citations. Citations are also written to the
#'   file as a side effect.
#' @export
#' @examples
#' write_pkg_bib(c("ggplot2", "gt"), file = tempfile(fileext = "bib"))
write_pkg_bib <- function(pkg, file, update = FALSE) {
  if (!file.exists(file)) {
    file.create(file)
  }

  # ensure package is installed
  if (update) {
    pak::pak(pkg, ask = FALSE)
  }

  lapply(pkg, cite_package) |>
    unlist() |>
    xfun::write_utf8(file)
}

cite_package <- function(x) {
  pkg_name <- stringr::str_replace(x, "^.*/", "")
  meta <- utils::packageDescription(pkg_name)
  repo <- meta$Repository

  pkg_cite <- if (is.null(repo) && meta$RemoteType == "github") {
    cite_github_pkg(meta)
  } else if (repo %in% c("CRAN", "RSPM")) {
    cite_cran_pkg(meta)
  }

  pkg_cite
}

cite_github_pkg <- function(meta) {
  authors <- pull_package_authors(meta)

  last_commit <- gh::gh(
    glue::glue(
      "/repos/{meta$RemoteUsername}/{meta$RemoteRepo}/commits/{meta$RemoteSha}"
    )
  )
  last_date <- last_commit$commit$author$date |>
    as.Date()

  url <- if (!is.null(meta$URL)) {
    meta$URL |>
      stringr::str_replace_all("\\n", " ") |>
      stringr::str_split_1(", ") |>
      stringr::str_subset("github")
  } else {
    NULL
  }

  glue::glue(
    "@manual{{R-{meta$Package},",
    "  author = {{{authors}}},",
    "  year = {{{stringr::str_sub(last_date, 1, 4)}}},",
    "  date = {{{last_date}}},",
    "  title = {{{{{meta$Package}}}: {format_pkg_title(meta$Title)}}},",
    "  version = {{R package version {meta$Version}}},",
    "  type = {{Computer software}},",
    "  {ifelse(is.null(url),
               glue::glue('note = {{Internal R Package}}'),
               glue::glue('url = {{{url}}}'))}",
    "}}",
    .sep = "\n"
  ) |>
    format_pkg_caps()
}

cite_cran_pkg <- function(meta) {
  authors <- pull_package_authors(meta)
  date <- meta$`Date/Publication` |>
    as.Date()

  glue::glue(
    "@manual{{R-{meta$Package},",
    "  author = {{{authors}}},",
    "  year = {{{stringr::str_sub(date, 1, 4)}}},",
    "  date = {{{date}}},",
    "  title = {{{{{meta$Package}}}: {format_pkg_title(meta$Title)}}},",
    "  version = {{R package version {meta$Version}}},",
    "  type = {{Computer software}},",
    "  publisher = {{The Comprehensive R Archive Network}},",
    "  doi = {{10.32614/CRAN.package.{meta$Package}}}",
    "}}",
    .sep = "\n"
  ) |>
    format_pkg_caps()
}

pull_package_authors <- function(meta) {
  authors <- meta$Author |>
    stringr::str_split_1(",\\n  ") |>
    stringr::str_subset("\\[aut") |>
    stringr::str_replace_all("\\n", "") |>
    stringr::str_replace_all(" \\[.*$", "") |>
    knitr::combine_words(sep = " and ", oxford_comma = FALSE)

  authors
}

format_pkg_title <- function(x) {
  stringr::str_replace_all(x, "\\n", " ")
}

format_pkg_caps <- function(citation) {
  citation |>
    stringr::str_replace_all(
      "Dynamic Learning Maps",
      "{Dynamic Learning Maps}"
    ) |>
    stringr::str_replace_all(
      "University of Kansas",
      "{University of Kansas}"
    ) |>
    stringr::str_replace_all("ATLAS", "{ATLAS}") |>
    stringr::str_replace_all("Taylor Swift's", "{Taylor Swift's}") |>
    stringr::str_replace_all(" R ", " {R} ") |>
    stringr::str_replace_all("Bayesian", "{Bayesian}") |>
    stringr::str_replace_all("(?<!m)Stan", "{Stan}") |>
    stringr::str_replace_all("TDCMs", "{TDCMs}")
}

taylor_meta <- utils::packageDescription("taylor")
wjake_meta <- utils::packageDescription("wjake")

test_that("authors are parsed correctly", {
  expect_equal(
    pull_package_authors(taylor_meta),
    "W. Jake Thompson",
    ignore_attr = TRUE
  )
})

test_that("package title is parsed correctly", {
  expect_equal(
    format_pkg_title(taylor_meta$Title),
    "Lyrics and Song Data for Taylor Swift's Discography",
    ignore_attr = TRUE
  )

  expect_equal(
    format_pkg_title(wjake_meta$Title),
    "Personal Themes and Formatting Preferences"
  )
})

test_that("capitalization is preserved", {
  taylor_cite <- cite_package("taylor")

  expect_match(taylor_cite, "\\@manual\\{R-taylor,")
  expect_match(taylor_cite, "author = \\{W. Jake Thompson\\}")
  expect_match(taylor_cite, "year = \\{[0-9]{4}\\}")
  expect_match(taylor_cite, "date = \\{[0-9]{4}-[0-9]{2}-[0-9]{2}\\}")
  expect_match(
    taylor_cite,
    paste0(
      "title = \\{\\{taylor\\}: Lyrics and Song Data for ",
      "\\{Taylor Swift's\\} Discography\\}"
    )
  )
  expect_match(taylor_cite, "version = \\{R package version [0-9\\.]{5}\\}")
  expect_match(taylor_cite, "type = \\{Computer software\\}")
  expect_match(
    taylor_cite,
    "publisher = \\{The Comprehensive \\{R\\} Archive Network\\}"
  )
  expect_match(taylor_cite, "doi = \\{10\\.32614\\/CRAN\\.package\\.taylor\\}")
})

test_that("bib file is written", {
  bib_file <- withr::local_tempfile(fileext = ".bib")
  write_pkg_bib("taylor", file = bib_file)

  expect_true(file.exists(bib_file))
  expect_equal(
    xfun::read_utf8(bib_file),
    stringr::str_split_1(cite_package("taylor"), "\n")
  )
})

test_that("github packages are citeable", {
  skip_on_cran()

  withr::local_envvar(
    R_USER_CACHE_DIR = tempfile()
  )

  bib_file <- withr::local_tempfile(fileext = ".bib")
  write_pkg_bib("wjakethompson/wjaketemplate", file = bib_file, update = TRUE)

  expect_true(file.exists(bib_file))
  expect_equal(
    xfun::read_utf8(bib_file),
    stringr::str_split_1(cite_package("wjaketemplate"), "\n")
  )
})

#' Palette color scale
#'
#' This is a qualitative scale using the color palette used for
#' https://wjakethompson.com. See [palette_wjake] for details.
#'
#' @param palette A character vector of the color palette to use in the scale
#'   (e.g., [palette_wjake], [palette_okabeito]).
#' @param order Numeric vector listing the order in which the colors should be
#'   used. Default is the order of the palette vector.
#' @param darken Relative amount by which the scale should be darkened (for
#'   positive values) or lightened (for negative values).
#' @param alpha Alpha transparency level of the color. Default is no
#'   transparency.
#' @param ... common discrete scale parameters: `name`, `breaks`, `labels`,
#'   `na.value`, `limits`, `guide`, and `aesthetics`. See [discrete_scale] for
#'   more details.
#'
#' @return A `ScaleDiscrete` object that can be added to a [ggplot2::ggplot()].
#' @export
#' @examples
#' library(ggplot2)
#'
#' ggplot(penguins, aes(x = bill_len, y = flipper_len, color = species)) +
#'   geom_point() +
#'   scale_color_wjake()
#'
#' ggplot(penguins, aes(bill_len, fill = species)) +
#'   geom_density(alpha = 0.7) +
#'   scale_fill_okabeito(order = c(1, 2, 5))
#'
#' @usage NULL
scale_palette <- function(
  aesthetics,
  palette,
  order = seq_along(palette),
  darken = 0,
  alpha = NA,
  ...
) {
  values <- palette[order]

  n <- length(values)
  darken <- rep_len(darken, n)
  alpha <- rep_len(alpha, n)

  di <- darken > 0
  if (sum(di) > 0) {
    values[di] <- colorspace::darken(values[di], amount = darken[di])
  }

  li <- darken < 0
  if (sum(li) > 0) {
    values[li] <- colorspace::lighten(values[li], amount = -1 * darken[li])
  }

  ai <- !is.na(alpha)
  if (sum(ai) > 0) {
    values[ai] <- scales::alpha(values[ai], alpha[ai])
  }

  pal <- function(n) {
    if (n > length(values)) {
      warning(
        "Insufficient values in manual scale. ",
        n,
        " needed but only ",
        length(values),
        " provided.",
        call. = FALSE
      )
    }
    values
  }
  ggplot2::discrete_scale(aesthetics, palette = pal, ...)
}

test_that("scale palette can be customized", {
  expect_true(is.environment(scale_palette(
    aesthetics = "fill",
    palette = palette_okabeito
  )))

  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    scale_palette(aesthetics = "color", palette = palette_okabeito)
  vdiffr::expect_doppelganger("default-scale", p)

  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    scale_palette(
      aesthetics = "color",
      palette = palette_okabeito,
      order = c(2, 4, 6),
      darken = 0.2
    )
  vdiffr::expect_doppelganger("darken-even", p)

  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    scale_palette(
      aesthetics = "color",
      palette = palette_okabeito,
      order = c(3, 5, 7),
      darken = -0.2,
      alpha = 0.6
    )
  vdiffr::expect_doppelganger("lighten-alpha-odd", p)

  p <- ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
    geom_point(aes(color = species), na.rm = TRUE) +
    scale_palette(aesthetics = "color", palette = palette_okabeito, order = 1:2)
  expect_warning(ggplot_build(p), "Insufficient values in manual scale.")
})

#' Create heading dividers
#'
#' Insert some hyphens padded to a specified console character width.
#'
#' @param width The width of console the hyphens should pad to
#'
#' @return A character string consisting of a ' ' and hyphens that fill to the
#'   specified `width`.
#'
#' @export
#' @examplesIf rstudioapi::isAvailable()
#' code_heading()
code_heading <- function(width = 80) {
  cur_location <- rstudioapi::primary_selection(
    rstudioapi::getSourceEditorContext()
  )

  cur_length <- cur_location$range$start[2]
  if (cur_length < (width - 2)) {
    rstudioapi::insertText(
      glue::glue(" {paste(rep('-', width - cur_length), collapse = '')}")
    )
  }
}

#' Write APA Words
#'
#' Confused about whether a number should be written out ("five") or use
#' numerals ("5")? Use this function! Most useful for R Markdown in-text
#' writing.
#'
#' @param x The number to be printed
#' @param ordinal Do you want the ordinal numbering (e.g., 1st, 6th, etc.)
#' @param negative The word used to indicate a negative number.
#'
#' @family formatters
#' @return A character string
#' @export
#'
#' @examples
#' apa_words(5)
#' apa_words(16)
#' apa_words(6, ordinal = TRUE)
apa_words <- function(x, ordinal = FALSE, negative = "negative") {
  check_number_whole(x)
  check_bool(ordinal)

  output <- if (ordinal) {
    scales::ordinal(x, big.mark = ",") |>
      stringr::str_replace("-", "\U2212")
  } else {
    if (x > -10 && x < 10) {
      english::words(x) |>
        stringr::str_replace("^minus", negative)
    } else {
      fmt_digits(x, digits = 0)
    }
  }

  output
}

#' Text and Number Formatting
#'
#' These formatting functions are used to format numerical values in a
#' consistent manner. This is useful for printing numbers inline with text, as
#' well as for formatting tables.
#'
#' @param x Number to be formatted.
#' @param digits Number of decimal places to retain.
#' @param big_mark Character used between every 3 digits to separate thousands.
#' @param min_value The minimum value `x` can take.
#' @param max_value The maximum value `x` can take.
#' @param sub_threshold The threshold to use when replacing value extremely
#'   close to `min_value` or `max_value`. By default, this is determined by the
#'   `digits` specified.
#' @param keep_boundary Whether to preserve true values of boundaries (i.e.,
#'   `min_value` and `max_value`). For example if `digits = 3`, `min_value = 0`,
#'   and `keep_boundary = FALSE`, a value exactly equal to 0 will become
#'   "<.001". If `keep_boundary = TRUE`, then a value of 0 will remain "0.000",
#'   and other small values (e.g., 0.000001) will continue to be replaced with
#'   "<.001".
#' @param ... Additional arguments passed to [scales::number_format()] or
#'   [fmt_digits()]. See Details for additional information.
#'
#' @details
#' [fmt_digits()] is a wrapper for [scales::number_format()] and prints a number
#' with the specified number of digits, suppressing values close to the minimum
#' and maximum values as necessary.
#'
#' Several helper functions are provided that wrap [fmt_digits()] with common
#' patterns of `min_value`, `max_value`, and `digits`.
#'
#' * [fmt_count()] is used for formatting integer values. Prints whole numbers
#'   with no decimals.
#' * [fmt_corr()] is used to format correlations or similar indices that are
#'   bounded between \[-1, 1\]. By default, these values report 3 decimal
#'   places, and the leading 0 is removed as required by APA (2020; section
#'   6.36).
#' * [fmt_prop()] is used to format proportions or similar indices that are
#'   bounded between \[0, 1\]. Similar to [fmt_corr()], leading 0s are removed.
#'   By default, 2 decimal places are reported.
#' * [fmt_pct()] is used to format percentage values that are bounded
#'   \[0, 100\]. By default, no decimal places are reported.
#' * [fmt_prop_pct()] formats proportions bounded by \[0, 1\] as percentages.
#'   That is, we first take `x * 100` and then apply [fmt_pct()].
#'
#' @name formatting
#' @family formatters
#' @return The updated character object of the same length as `x`.
#'
#' @export
#' @examples
#' fmt_digits(runif(5, min = 5, max = 15), digits = 1)
#' fmt_digits(runif(5, min = 5, max = 15), digits = 3)
#'
#' fmt_count(sample(10000:99999, size = 5))
#'
#' fmt_corr(runif(5, min = -1, max = 1))
#'
#' fmt_prop(runif(5, min = 0, max = 1))
#'
#' fmt_pct(runif(5, min = 0, max = 100))
#'
#' fmt_prop_pct(runif(5, min = 0, max = 1))
#'
#' @references American Psychological Association. (2020). *Publication manual
#'   of the American Psychological Association* (7th ed.).
#'   \doi{doi:10.1037/0000165-000}
fmt_digits <- function(
  x,
  digits = 1,
  big_mark = ",",
  min_value = -Inf,
  max_value = Inf,
  sub_threshold = 1 / (10^digits),
  keep_boundary = FALSE,
  ...
) {
  to_print <- scales::number(
    x,
    accuracy = 1 / (10^digits),
    big.mark = big_mark,
    style_negative = "minus",
    ...
  )

  if (!is.infinite(min_value)) {
    min_print <- scales::number(
      min_value + sub_threshold,
      accuracy = 1 / (10^digits),
      big.mark = big_mark,
      style_negative = "minus"
    )
    if (keep_boundary) {
      to_print[x < (min_value + sub_threshold) & x != min_value] <- paste0(
        "<",
        min_print
      )
    } else {
      to_print[x < (min_value + sub_threshold)] <- paste0(
        "<",
        min_print
      )
    }
  }

  if (!is.infinite(max_value)) {
    max_print <- scales::number(
      max_value - sub_threshold,
      accuracy = 1 / (10^digits),
      big.mark = big_mark,
      style_negative = "minus"
    )
    if (keep_boundary) {
      to_print[x > (max_value - sub_threshold) & x != max_value] <- paste0(
        ">",
        max_print
      )
    } else {
      to_print[x > (max_value - sub_threshold)] <- paste0(
        ">",
        max_print
      )
    }
  }

  to_print
}

#' @rdname formatting
#' @export
fmt_count <- function(x, ...) {
  for (i in seq_along(x)) {
    check_number_whole(x[i])
  }

  fmt_digits(x, digits = 0, min_value = 0, ...)
}

#' @rdname formatting
#' @export
fmt_corr <- function(x, digits = 3, ...) {
  for (i in seq_along(x)) {
    check_number_decimal(x[i], min = -1, max = 1)
  }

  fmt_digits(
    x,
    digits = digits,
    min_value = -1,
    max_value = 1,
    ...
  ) |>
    stringr::str_replace(stringr::fixed("0."), ".")
}

#' @rdname formatting
#' @export
fmt_prop <- function(x, digits = 2, ...) {
  for (i in seq_along(x)) {
    check_number_decimal(x[i], min = 0, max = 1)
  }

  fmt_digits(
    x,
    digits = digits,
    min_value = 0,
    max_value = 1,
    ...
  ) |>
    stringr::str_replace(stringr::fixed("0."), ".")
}

#' @rdname formatting
#' @export
fmt_pct <- function(x, digits = 0, ...) {
  for (i in seq_along(x)) {
    check_number_decimal(x[i], min = 0, max = 100)
  }

  fmt_digits(
    x,
    digits = digits,
    min_value = 0,
    max_value = 100,
    ...
  )
}

#' @rdname formatting
#' @export
fmt_prop_pct <- function(x, digits = 0, ...) {
  for (i in seq_along(x)) {
    check_number_decimal(x[i], min = 0, max = 1)
  }

  fmt_digits(
    x * 100,
    digits = digits,
    min_value = 0,
    max_value = 100,
    ...
  )
}

test_that("apa_words() correctly formats", {
  expect_equal(apa_words(-10), stringr::str_replace("-10", "-", "\U2212"))
  expect_equal(apa_words(-9), "negative nine")
  expect_equal(apa_words(-5, negative = "minus"), "minus five")
  expect_equal(apa_words(-1, negative = "blue"), "blue one")
  expect_equal(apa_words(0), "zero")
  expect_equal(apa_words(2), "two")
  expect_equal(apa_words(9), "nine")
  expect_equal(apa_words(10), "10")

  expect_equal(apa_words(16, ordinal = TRUE), "16th")
  expect_equal(apa_words(12, ordinal = TRUE), "12th")
  expect_equal(apa_words(23, ordinal = TRUE), "23rd")
  expect_equal(apa_words(-13, ordinal = TRUE), gsub("-", "\U2212", "-13th"))
})

test_that("fmt_digits() uses the correct number of decimals", {
  rand <- runif(n = 10, 0, 100)
  expect_equal(fmt_digits(rand, digits = 2), sprintf("%0.2f", rand))
  expect_equal(fmt_digits(rand, digits = 4), sprintf("%0.4f", rand))
})

test_that("fmt_digits() correctly inserts commas", {
  rand <- runif(n = 100, 100, 1000000)
  expect_equal(
    fmt_digits(rand, digits = 2),
    formatC(rand, digits = 2, big.mark = ",", format = "f")
  )
  expect_equal(
    fmt_digits(rand, digits = 1),
    formatC(rand, digits = 1, big.mark = ",", format = "f")
  )
})

test_that("fmt_digits() acknowledges minimum and maximum values", {
  rand <- c(0, 0.01, 0.1, sort(runif(14, min = .1, max = .9)), 0.9, 0.99, 1)
  expect_equal(
    fmt_digits(rand, digits = 2, min_value = 0, max_value = 1),
    c("<0.01", sprintf("%0.2f", rand[2:19]), ">0.99")
  )
  expect_equal(
    fmt_digits(rand, digits = 1, min_value = 0, max_value = 1),
    c("<0.1", "<0.1", sprintf("%0.1f", rand[3:18]), ">0.9", ">0.9")
  )
})

test_that("fmt_digits() can use a different threshold for subbing", {
  rand <- c(
    runif(5, min = 0, max = 5),
    runif(10, min = 5, max = 15),
    runif(5, min = 15, max = 20)
  )
  expect_equal(
    fmt_digits(rand, digits = 1, min_value = 0, sub_threshold = 5),
    c(rep("<5.0", 5), sprintf("%0.1f", rand[6:20]))
  )
  expect_equal(
    fmt_digits(rand, digits = 1, max_value = 20, sub_threshold = 5),
    c(sprintf("%0.1f", rand[1:15]), rep(">15.0", 5))
  )
  expect_equal(
    fmt_digits(
      rand,
      digits = 1,
      min_value = 0,
      max_value = 20,
      sub_threshold = 5
    ),
    c(rep("<5.0", 5), sprintf("%0.1f", rand[6:15]), rep(">15.0", 5))
  )
})

test_that("fmt_digits() can keep boundaries if needed", {
  rand <- c(
    0,
    runif(4, min = 0, max = 5),
    runif(10, min = 5, max = 15),
    runif(4, min = 15, max = 20),
    20
  )
  expect_equal(
    fmt_digits(
      rand,
      digits = 1,
      min_value = 0,
      sub_threshold = 5,
      keep_boundary = TRUE
    ),
    c("0.0", rep("<5.0", 4), sprintf("%0.1f", rand[6:20]))
  )
  expect_equal(
    fmt_digits(
      rand,
      digits = 1,
      max_value = 20,
      sub_threshold = 5,
      keep_boundary = TRUE
    ),
    c(sprintf("%0.1f", rand[1:15]), rep(">15.0", 4), "20.0")
  )
  expect_equal(
    fmt_digits(
      rand,
      digits = 1,
      min_value = 0,
      max_value = 20,
      sub_threshold = 5,
      keep_boundary = TRUE
    ),
    c(
      "0.0",
      rep("<5.0", 4),
      sprintf("%0.1f", rand[6:15]),
      rep(">15.0", 4),
      "20.0"
    )
  )
})

test_that("fmt_count() adds comma and has no decimals", {
  rand <- sample(0:1000000, size = 100000)
  expect_equal(
    fmt_count(rand, sub_threshold = 0),
    formatC(rand, digits = 0, big.mark = ",", format = "f")
  )
})

test_that("fmt_count() can suppress small values", {
  rand <- seq(0, 10000, by = 100)
  expect_equal(
    fmt_count(rand, sub_threshold = 900),
    c(
      rep("<900", 9),
      formatC(rand[10:101], digits = 0, big.mark = ",", format = "f")
    )
  )
})

test_that("fmt_corr() has correct digits and removes leading 0", {
  rand <- sort(runif(50, min = -0.99, max = 0.99))
  expect_equal(
    fmt_corr(rand),
    stringr::str_replace(sprintf("%0.3f", rand), "0\\.", ".") |>
      stringr::str_replace("-\\.000", ".000") |>
      stringr::str_replace("-", "\U2212")
  )
  expect_equal(
    fmt_corr(rand, digits = 2),
    stringr::str_replace(sprintf("%0.2f", rand), "0\\.", ".") |>
      stringr::str_replace("-\\.00", ".00") |>
      stringr::str_replace("-", "\U2212")
  )
})

test_that("fmt_corr() respects boundaries and thresholds", {
  rand <- c(-1, -.999, -.99, -.9, sort(runif(12, -.9, .9)), .9, .99, .999, 1)

  rand3 <- stringr::str_replace(sprintf("%0.3f", rand), "0\\.", ".") |>
    stringr::str_replace("-", "\U2212")
  expect_equal(
    fmt_corr(rand, digits = 3),
    c("<-.999", rand3[2:19], ">.999") |>
      stringr::str_replace("-\\.000", ".000") |>
      stringr::str_replace("-", "\U2212")
  )
  expect_equal(
    fmt_corr(rand, digits = 3, keep_boundary = TRUE),
    c("-1.000", rand3[2:19], "1.000") |>
      stringr::str_replace("-\\.000", ".000") |>
      stringr::str_replace("-", "\U2212")
  )
  expect_equal(
    fmt_corr(rand, digits = 3, sub_threshold = .1),
    c(rep("<-.900", 3), rand3[4:17], rep(">.900", 3)) |>
      stringr::str_replace("-\\.000", ".000") |>
      stringr::str_replace("-", "\U2212")
  )

  rand2 <- stringr::str_replace(sprintf("%0.2f", rand), "0\\.", ".") |>
    stringr::str_replace("-\\.00", ".00") |>
    stringr::str_replace("-", "\U2212")
  expect_equal(
    fmt_corr(rand, digits = 2),
    c("<-.99", "<-.99", rand2[3:18], ">.99", ">.99") |>
      stringr::str_replace("-\\.00", ".00") |>
      stringr::str_replace("-", "\U2212")
  )
  expect_equal(
    fmt_corr(rand, digits = 2, keep_boundary = TRUE),
    c("-1.00", "<-.99", rand2[3:18], ">.99", "1.00") |>
      stringr::str_replace("-\\.00", ".00") |>
      stringr::str_replace("-", "\U2212")
  )
})

test_that("fmt_prop() has correct digits and removes leading 0", {
  rand <- sort(runif(50, min = 0.1, max = 0.9))
  expect_equal(
    fmt_prop(rand),
    stringr::str_replace(sprintf("%0.2f", rand), "0\\.", ".")
  )
  expect_equal(
    fmt_prop(rand, digits = 3),
    stringr::str_replace(sprintf("%0.3f", rand), "0\\.", ".")
  )
})

test_that("fmt_prop() respects boundaries and thresholds", {
  rand <- c(0, 0.001, 0.01, 0.1, runif(12, .1, .9), 0.9, 0.99, 0.999, 1)
  expect_equal(
    fmt_prop(rand, digits = 3),
    c(
      rep("<.001", 1),
      stringr::str_replace(sprintf("%0.3f", rand[2:19]), "0\\.", "."),
      rep(">.999", 1)
    )
  )
  expect_equal(
    fmt_prop(rand, digits = 3, keep_boundary = TRUE),
    c(
      rep(".000", 1),
      stringr::str_replace(sprintf("%0.3f", rand[2:19]), "0\\.", "."),
      rep("1.000", 1)
    )
  )
  expect_equal(
    fmt_prop(rand, digits = 3, sub_threshold = .1, keep_boundary = TRUE),
    c(
      ".000",
      "<.100",
      "<.100",
      stringr::str_replace(sprintf("%0.3f", rand[4:17]), "0\\.", "."),
      ">.900",
      ">.900",
      "1.000"
    )
  )

  expect_equal(
    fmt_prop(rand),
    c(
      rep("<.01", 2),
      stringr::str_replace(sprintf("%0.2f", rand[3:18]), "0\\.", "."),
      rep(">.99", 2)
    )
  )
  expect_equal(
    fmt_prop(rand, keep_boundary = TRUE),
    c(
      ".00",
      "<.01",
      stringr::str_replace(sprintf("%0.2f", rand[3:18]), "0\\.", "."),
      ">.99",
      "1.00"
    )
  )
  expect_equal(
    fmt_prop(rand, sub_threshold = .1, keep_boundary = FALSE),
    c(
      "<.10",
      "<.10",
      "<.10",
      stringr::str_replace(sprintf("%0.2f", rand[4:17]), "0\\.", "."),
      ">.90",
      ">.90",
      ">.90"
    )
  )
})

test_that("fmt_pct() has correct digits", {
  rand <- sort(runif(50, min = 1, max = 99))
  expect_equal(fmt_pct(rand), sprintf("%0.0f", rand))
  expect_equal(fmt_pct(rand, digits = 2), sprintf("%0.2f", rand))
})

test_that("fmt_pct() respects boundaries and thresholds", {
  rand <- c(0, 0.01, 0.1, 1, sort(runif(42, 1, 99)), 99, 99.9, 99.99, 100)
  expect_equal(
    fmt_pct(rand),
    c(rep("<1", 3), sprintf("%0.0f", rand[4:47]), rep(">99", 3))
  )
  expect_equal(
    fmt_pct(rand, keep_boundary = TRUE),
    c("0", rep("<1", 2), sprintf("%0.0f", rand[4:47]), rep(">99", 2), "100")
  )

  expect_equal(
    fmt_pct(rand, digits = 1),
    c(rep("<0.1", 2), sprintf("%0.1f", rand[3:48]), rep(">99.9", 2))
  )
  expect_equal(
    fmt_pct(rand, digits = 1, keep_boundary = TRUE),
    c("0.0", "<0.1", sprintf("%0.1f", rand[3:48]), ">99.9", "100.0")
  )
  expect_equal(
    fmt_pct(rand, digits = 1, sub_threshold = 1),
    c(rep("<1.0", 3), sprintf("%0.1f", rand[4:47]), rep(">99.0", 3))
  )
})

test_that("fmt_prop_pct() has correct digits", {
  rand <- sort(runif(50, min = 0.01, max = 0.99))
  expect_equal(fmt_prop_pct(rand), sprintf("%0.0f", rand * 100))
  expect_equal(fmt_prop_pct(rand, digits = 2), sprintf("%0.2f", rand * 100))
})

test_that("fmt_prop_pct() respects boundaries and thresholds", {
  rand <- c(
    0,
    0.0001,
    0.001,
    0.01,
    sort(runif(42, 0.01, 0.99)),
    0.99,
    0.999,
    0.9999,
    1
  )
  expect_equal(
    fmt_prop_pct(rand),
    c(rep("<1", 3), sprintf("%0.0f", rand[4:47] * 100), rep(">99", 3))
  )
  expect_equal(
    fmt_prop_pct(rand, keep_boundary = TRUE),
    c(
      "0",
      rep("<1", 2),
      sprintf("%0.0f", rand[4:47] * 100),
      rep(">99", 2),
      "100"
    )
  )

  expect_equal(
    fmt_prop_pct(rand, digits = 1),
    c(rep("<0.1", 2), sprintf("%0.1f", rand[3:48] * 100), rep(">99.9", 2))
  )
  expect_equal(
    fmt_prop_pct(rand, digits = 1, keep_boundary = TRUE),
    c("0.0", "<0.1", sprintf("%0.1f", rand[3:48] * 100), ">99.9", "100.0")
  )
  expect_equal(
    fmt_prop_pct(rand, digits = 1, sub_threshold = 1),
    c(rep("<1.0", 3), sprintf("%0.1f", rand[4:47] * 100), rep(">99.0", 3))
  )
})

#' Custom color ramps
#'
#' These color ramp functions create color scales that can be used for making
#' ggplot2 plots and gt tables. Color ramps are based on the [palette_wjake]
#' color palette.
#'
#' @inheritParams grDevices::colorRamp
#' @param output Colors to pull from the color ramp. Numbers range from 0-1,
#'   which is a normalized sliding scale of the color ramp.
#' @param end The end color that the base color should fade into.
#' @param mid For pre-made diverging palettes, the color of the midpoint of the
#'   scale.
#'
#' @details
#' [make_color_pal()] can be used to create a color ramp function for any set of
#' valid colors.
#'
#' [ramp_blue()], [ramp_yellow()], and [ramp_yelblu()] are pre-made color ramps
#' based on the blue and yellow colors from the [palette_wjake] color palette.
#'
#' [ramp_orgblu()] is a pre-made color ramp based on the first two colors from
#' the [palette_okabeito] color palette.
#'
#' @return [make_color_pal()] returns a function that accepts a numeric vector
#'   of values between 0 and 1 and returns a character vector of hex color
#'   codes. [ramp_blue()], [ramp_yellow()], [ramp_yelblu()], and [ramp_orgblu()]
#'   return a character vector of hex color codes.
#' @name color_ramp

#' @rdname color_ramp
#' @export
#' @examples
#' new_ramp <- make_color_pal(c("red", "grey", "blue"))
#' new_ramp(seq(0, 1, length.out = 10))
#'
#' # Pre-made palettes
#' ramp_blue(seq(0, 1, by = 0.2))
#' ramp_yellow(seq(0.2, 1, length.out = 5))
make_color_pal <- function(colors, bias = 1) {
  get_color <- grDevices::colorRamp(colors, bias = bias)
  function(x) grDevices::rgb(get_color(x), maxColorValue = 255)
}

#' @rdname color_ramp
#' @export
ramp_blue <- function(output, end = "#FFFFFF") {
  ramp <- make_color_pal(c(end, "#009FB7"), bias = 1)
  ramp(output)
}

#' @rdname color_ramp
#' @export
ramp_yellow <- function(output, end = "#FFFFFF") {
  ramp <- make_color_pal(c(end, "#FED766"), bias = 1)
  ramp(output)
}

#' @rdname color_ramp
#' @export
ramp_yelblu <- function(output, mid = "#F0F0F0") {
  ramp <- make_color_pal(c("#FED766", mid, "#009FB7"), bias = 1)
  ramp(output)
}

#' @rdname color_ramp
#' @export
ramp_orgblu <- function(output, mid = "#999999") {
  ramp <- make_color_pal(c("#E69F00", mid, "#56B4E9"), bias = 1)
  ramp(output)
}

test_that("make_color_pal() produces a function", {
  expect_true(is.function(make_color_pal(c("red", "blue"))))

  func <- make_color_pal(c("orange", "grey", "green"))
  expect_true(is.function(func))
  expect_equal(formalArgs(func), "x")
})

test_that("prebuilt ramps work", {
  # blue -----
  expect_equal(ramp_blue(c(0, 1)), c("#FFFFFF", "#009FB7"))
  expect_equal(
    ramp_blue(seq(0, 1, by = .2)),
    c("#FFFFFF", "#CCEBF0", "#99D8E2", "#65C5D3", "#32B2C5", "#009FB7")
  )
  expect_equal(
    ramp_blue(seq(.2, 1, by = .2)),
    c("#CCEBF0", "#99D8E2", "#65C5D3", "#32B2C5", "#009FB7")
  )
  expect_equal(
    ramp_blue(seq(0, 1, length.out = 10), end = "#009FB7"),
    rep("#009FB7", 10)
  )

  # yellow -----
  expect_equal(ramp_yellow(c(0, 1)), c("#FFFFFF", "#FED766"))
  expect_equal(
    ramp_yellow(seq(0, 1, by = .2)),
    c("#FFFFFF", "#FEF7E0", "#FEEFC1", "#FEE7A3", "#FEDF84", "#FED766")
  )
  expect_equal(
    ramp_yellow(seq(.2, 1, by = .2)),
    c("#FEF7E0", "#FEEFC1", "#FEE7A3", "#FEDF84", "#FED766")
  )
  expect_equal(
    ramp_yellow(seq(0, 1, length.out = 10), end = "#FED766"),
    rep("#FED766", 10)
  )

  # yellow-blue -----
  expect_equal(ramp_yelblu(c(0, 1)), c("#FED766", "#009FB7"))
  expect_equal(ramp_yelblu(c(0, 0.5, 1)), c("#FED766", "#F0F0F0", "#009FB7"))
  expect_equal(
    ramp_yelblu(seq(0, 1, by = 0.25)),
    c("#FED766", "#F7E3AB", "#F0F0F0", "#78C7D3", "#009FB7")
  )
  expect_equal(
    ramp_yelblu(seq(0.25, 1, by = 0.25)),
    c("#F7E3AB", "#F0F0F0", "#78C7D3", "#009FB7")
  )
  expect_equal(
    ramp_yelblu(seq(0.25, 0.75, by = 0.25)),
    c("#F7E3AB", "#F0F0F0", "#78C7D3")
  )

  # orange-blue -----
  expect_equal(ramp_orgblu(c(0, 1)), c("#E69F00", "#56B4E9"))
  expect_equal(ramp_orgblu(c(0, 0.5, 1)), c("#E69F00", "#999999", "#56B4E9"))
  expect_equal(
    ramp_orgblu(seq(0, 1, by = 0.25)),
    c("#E69F00", "#BF9C4C", "#999999", "#77A6C1", "#56B4E9")
  )
  expect_equal(
    ramp_orgblu(seq(0.25, 1, by = 0.25)),
    c("#BF9C4C", "#999999", "#77A6C1", "#56B4E9")
  )
  expect_equal(
    ramp_orgblu(seq(0.25, 0.75, by = 0.25), mid = "white"),
    c("#F2CF7F", "#FFFFFF", "#AAD9F3")
  )
})

#' Personalized gt theme
#'
#' A custom theme for tables generated with [gt::gt()].
#'
#' @inheritParams gt_theme_apa
#' @param ... Additional arguments passed to [gt_theme_apa()] and
#'   [gt::tab_options()].
#'
#' @return An object of class `gt_tbl`.
#' @export
#' @examples
#' gt::gt(head(penguins)) |>
#'   gt_theme_wjake()
gt_theme_wjake <- function(data, bg_color = "#F0F0F0", font_size = 16, ...) {
  data |>
    gt::tab_source_note("Table: @wjakethompson.com") |>
    gt_theme_apa(
      font = "Source Sans Pro",
      bg_color = bg_color,
      font_size = font_size,
      ...
    ) |>
    gt::opt_all_caps(locations = "column_labels") |>
    gt::tab_options(
      table_body.hlines.style = "solid",
      table.border.top.style = "solid",
      table.border.top.color = "#A8A8A8",
      table.border.top.width = gt::px(1),
      table.border.bottom.style = "solid",
      table.border.bottom.color = "#A8A8A8",
      table.border.bottom.width = gt::px(1),
      table_body.border.bottom.style = "solid",
      table_body.border.bottom.width = gt::px(2),
      table_body.border.bottom.color = "#D3D3D3",
      column_labels.border.top.style = "none",
      column_labels.border.bottom.style = "solid",
      column_labels.border.bottom.width = gt::px(3),
      column_labels.border.bottom.color = "black",
      heading.border.bottom.style = "none",
      heading.align = "left",
      source_notes.padding = gt::px(10),
      source_notes.border.lr.width = gt::px(0),
      data_row.padding = gt::px(3)
    ) |>
    gt::tab_style(
      style = list(gt::cell_text(transform = "uppercase", align = "right")),
      locations = gt::cells_source_notes()
    ) |>
    gt::tab_style(
      style = list(gt::cell_text(weight = "bold")),
      locations = gt::cells_title()
    )
}

#' Create an announcement adjective
#'
#' @return A character string
#' @export
#'
#' @examples
#' announce()
announce <- function() {
  phrases <- list(
    chuffed = c(""),
    pleased = c("", "most", "very", "extremely", "well"),
    stoked = c(""),
    chuffed = c("", "very"),
    happy = c("", "so", "very", "exceedingly"),
    thrilled = c(""),
    delighted = c(""),
    `tickled pink` = c(""),
    ecstatic = c(""),
    elated = c(""),
    excited = c("", "remarkably"),
    euphoric = c(""),
    overjoyed = c("")
  )
  i <- sample(length(phrases), 1)
  word <- names(phrases)[[i]]
  modifier <- sample(phrases[[i]], 1)
  paste0(modifier, if (modifier != "") " ", word)
}

test_that("announce() creates phrases", {
  phrases <- vector(mode = "character", length = 20)
  for (i in seq_along(phrases)) {
    phrases[i] <- announce()
    expect_true(is.character(phrases[i]))
    expect_false(is.na(phrases[i]))
    expect_false(phrases[i] == "")
    expect_true(nchar(phrases[i]) > 0)
  }

  expect_true(length(unique(phrases)) > 1)
})

#' X & Y scales for percent and comma labels
#'
#' Wrappers around [ggplot2::continuous_scale()] that provide automatic percent
#' or comma formatting.
#'
#' @param ... Additional arguments passed to [ggplot2::scale_x_continuous()] or
#'   [ggplot2::scale_y_continuous()].
#'
#' @returns A ggplot2 continuous scale.
#'
#' @name xy_scales
#' @export
#' @examples
#' library(ggplot2)
#' set.seed(1234)
#'
#' ggplot() +
#'   geom_point(aes(x = sample(1000:9999, size = 100), y = runif(100, 0, 1))) +
#'   scale_x_comma() +
#'   scale_y_percent(breaks = seq(0, 1, by = .2))
scale_x_comma <- function(...) {
  scale_x_continuous(labels = scales::label_comma(), ...)
}

#' @rdname xy_scales
#' @export
scale_y_comma <- function(...) {
  scale_y_continuous(labels = scales::label_comma(), ...)
}

#' @rdname xy_scales
#' @export
scale_x_percent <- function(...) {
  scale_x_continuous(labels = scales::label_percent(), ...)
}

#' @rdname xy_scales
#' @export
scale_y_percent <- function(...) {
  scale_y_continuous(labels = scales::label_percent(), ...)
}

test_that("xy scales render correctly", {
  example_data <- data.frame(
    pct = seq(0, 1, length.out = 50),
    n = seq(0, 5000, length.out = 50)
  )

  p <- ggplot(example_data, aes(x = n, y = n)) +
    geom_point() +
    scale_x_comma() +
    scale_y_comma()
  vdiffr::expect_doppelganger("comma-comma", p)

  p <- ggplot(example_data, aes(x = n, y = pct)) +
    geom_point() +
    scale_x_comma() +
    scale_y_percent()
  vdiffr::expect_doppelganger("comma-percent", p)

  p <- ggplot(example_data, aes(x = pct, y = n)) +
    geom_point() +
    scale_x_percent() +
    scale_y_comma()
  vdiffr::expect_doppelganger("percent-comma", p)

  p <- ggplot(example_data, aes(x = pct, y = pct)) +
    geom_point() +
    scale_x_comma() +
    scale_y_comma()
  vdiffr::expect_doppelganger("percent-percent", p)
})

# gt_theme_apa() formats integers with 0 decimals

    Code
      gt::extract_body(tbl)
    Output
      # A tibble: 3 x 1
        x    
        <chr>
      1   1  
      2  10  
      3 100  

# gt_theme_apa() formats doubles with dec_dig decimals

    Code
      gt::extract_body(tbl1)
    Output
      # A tibble: 2 x 1
        x    
        <chr>
      1  1.2 
      2 56.8 

---

    Code
      gt::extract_body(tbl2)
    Output
      # A tibble: 2 x 1
        x    
        <chr>
      1  1.23
      2 56.79

# gt_theme_apa() has correlations with 3 digits and no leading zero

    Code
      gt::extract_body(tbl)
    Output
      # A tibble: 4 x 1
        r    
        <chr>
      1 −.500
      2  .000
      3  .123
      4  .456

# gt_theme_apa() substitutes extreme doubles based on dec_dig

    Code
      gt::extract_body(tbl1)
    Output
      # A tibble: 4 x 1
        x       
        <chr>   
      1  &lt;0.1
      2      0.1
      3 &gt;99.9
      4 &gt;99.9

---

    Code
      gt::extract_body(tbl2)
    Output
      # A tibble: 4 x 1
        x    
        <chr>
      1  0.05
      2  0.10
      3 99.90
      4 99.95

# gt_theme_apa() substitutes extreme correlations based on corr_dig

    Code
      gt::extract_body(tbl1)
    Output
      # A tibble: 4 x 1
        r        
        <chr>    
      1 &lt;−.999
      2 &lt;−.999
      3  &gt;.999
      4  &gt;.999

---

    Code
      gt::extract_body(tbl2)
    Output
      # A tibble: 4 x 1
        r       
        <chr>   
      1 &lt;−.99
      2 &lt;−.99
      3  &gt;.99
      4  &gt;.99

# gt_theme_apa() can skip extreme substitution

    Code
      gt::extract_body(tbl)
    Output
      # A tibble: 2 x 2
        x     r     
        <chr> <chr> 
      1   0.1 −1.000
      2 100.0  1.000

# gt_theme_apa() applies number formatting in grouped tables

    Code
      gt::extract_body(tbl)
    Output
      # A tibble: 3 x 4
        `::group_id::` int_col dbl_col  corr_col
        <chr>          <chr>   <chr>    <chr>   
      1 A              1            1.5 .123    
      2 A              2           20.0 .456    
      3 B              3       &gt;99.9 .789    

# gt_theme_wjake() inherits gt_theme_apa() number formatting

    Code
      gt::extract_body(tbl)
    Output
      # A tibble: 2 x 3
        int_col dbl_col corr_col
        <chr>   <chr>   <chr>   
      1 1        1.5    .123    
      2 2       20.0    .456    

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{code_heading}
\alias{code_heading}
\title{Create heading dividers}
\usage{
code_heading(width = 80)
}
\arguments{
\item{width}{The width of console the hyphens should pad to}
}
\value{
A character string consisting of a ' ' and hyphens that fill to the
specified \code{width}.
}
\description{
Insert some hyphens padded to a specified console character width.
}
\examples{
\dontshow{if (rstudioapi::isAvailable()) withAutoprint(\{ # examplesIf}
code_heading()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-scale-xy.R
\name{xy_scales}
\alias{xy_scales}
\alias{scale_x_comma}
\alias{scale_y_comma}
\alias{scale_x_percent}
\alias{scale_y_percent}
\title{X & Y scales for percent and comma labels}
\usage{
scale_x_comma(...)

scale_y_comma(...)

scale_x_percent(...)

scale_y_percent(...)
}
\arguments{
\item{...}{Additional arguments passed to \code{\link[ggplot2:scale_continuous]{ggplot2::scale_x_continuous()}} or
\code{\link[ggplot2:scale_continuous]{ggplot2::scale_y_continuous()}}.}
}
\value{
A ggplot2 continuous scale.
}
\description{
Wrappers around \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale()}} that provide automatic percent
or comma formatting.
}
\examples{
library(ggplot2)
set.seed(1234)

ggplot() +
  geom_point(aes(x = sample(1000:9999, size = 100), y = runif(100, 0, 1))) +
  scale_x_comma() +
  scale_y_percent(breaks = seq(0, 1, by = .2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wjake-package.R
\docType{package}
\name{wjake-package}
\alias{wjake}
\alias{wjake-package}
\title{wjake: A package for package installation and miscellaneous functions}
\description{
Personal package for helping with R setup and random tasks that I Google too
often.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/wjakethompson/wjake}
  \item \url{https://wjake.wjakethompson.com}
  \item Report bugs at \url{https://github.com/wjakethompson/wjake/issues}
}

}
\author{
\strong{Maintainer}: W. Jake Thompson \email{wjakethompson@gmail.com} (\href{https://orcid.org/0000-0001-7339-0300}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fmt-text.R
\name{apa_words}
\alias{apa_words}
\title{Write APA Words}
\usage{
apa_words(x, ordinal = FALSE, negative = "negative")
}
\arguments{
\item{x}{The number to be printed}

\item{ordinal}{Do you want the ordinal numbering (e.g., 1st, 6th, etc.)}

\item{negative}{The word used to indicate a negative number.}
}
\value{
A character string
}
\description{
Confused about whether a number should be written out ("five") or use
numerals ("5")? Use this function! Most useful for R Markdown in-text
writing.
}
\examples{
apa_words(5)
apa_words(16)
apa_words(6, ordinal = TRUE)
}
\seealso{
Other formatters: 
\code{\link{formatting}}
}
\concept{formatters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors-wjake.R
\docType{data}
\name{palette_wjake}
\alias{palette_wjake}
\alias{scale_colour_wjake}
\alias{scale_color_wjake}
\alias{scale_fill_wjake}
\title{Personal color palette}
\format{
A character vector of 5 hex color codes.
}
\usage{
palette_wjake
}
\description{
Color palette used for theming https://wjakethompson.com. The palette also
includes color scales that can be used in place of
\code{\link[ggplot2:scale_colour_discrete]{ggplot2::scale_color_discrete()}} or \code{\link[ggplot2:scale_colour_discrete]{ggplot2::scale_fill_discrete()}}.
}
\examples{
scales::show_col(palette_wjake)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors-okabeito.R
\docType{data}
\name{palette_okabeito}
\alias{palette_okabeito}
\alias{scale_colour_okabeito}
\alias{scale_color_okabeito}
\alias{scale_fill_okabeito}
\title{Okabe-Ito color palette}
\format{
A character vector of 8 hex color codes.
}
\usage{
palette_okabeito
}
\description{
Colorblind friendly palette taken originally described by Okabe and Ito
(2008). The original palette has been modified slightly to make the yellow
more visible on a white background, as described by Lüdecke et al. (2021).
The palette also includes color scales that can be used in place of
\code{\link[ggplot2:scale_colour_discrete]{ggplot2::scale_color_discrete()}} or \code{\link[ggplot2:scale_colour_discrete]{ggplot2::scale_fill_discrete()}}.
}
\examples{
scales::show_col(palette_okabeito)
}
\references{
Lüdecke, D., Patil, I., Ben-Shachar, M. S., Wiernik, B. M.,
Waggoner, P., & Makowski, D. (2021). see: An R package for visualizing
statistical models. \emph{Journal of Open Source Software, 6}(64), Article 3393.
\doi{10.21105/joss.03393}

Okabe, M., & Ito, K. (2008). Color universal design (CUD): How to
make figures and presentations that are friendly to colorblind people.
\url{https://jfly.uni-koeln.de/color/#pallet} (Original work published 2002)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gt-theme-wjake.R
\name{gt_theme_wjake}
\alias{gt_theme_wjake}
\title{Personalized gt theme}
\usage{
gt_theme_wjake(data, bg_color = "#F0F0F0", font_size = 16, ...)
}
\arguments{
\item{data}{The gt table data object.}

\item{bg_color}{The background color of the table.}

\item{font_size}{The base font size for the table.}

\item{...}{Additional arguments passed to \code{\link[=gt_theme_apa]{gt_theme_apa()}} and
\code{\link[gt:tab_options]{gt::tab_options()}}.}
}
\value{
An object of class \code{gt_tbl}.
}
\description{
A custom theme for tables generated with \code{\link[gt:gt]{gt::gt()}}.
}
\examples{
gt::gt(head(penguins)) |>
  gt_theme_wjake()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors-ramps.R
\name{color_ramp}
\alias{color_ramp}
\alias{make_color_pal}
\alias{ramp_blue}
\alias{ramp_yellow}
\alias{ramp_yelblu}
\alias{ramp_orgblu}
\title{Custom color ramps}
\usage{
make_color_pal(colors, bias = 1)

ramp_blue(output, end = "#FFFFFF")

ramp_yellow(output, end = "#FFFFFF")

ramp_yelblu(output, mid = "#F0F0F0")

ramp_orgblu(output, mid = "#999999")
}
\arguments{
\item{colors}{colors to interpolate; must be a valid argument to
    \code{\link[grDevices]{col2rgb}()}.}

\item{bias}{a positive number.  Higher values give more widely spaced
    colors at the high end.}

\item{output}{Colors to pull from the color ramp. Numbers range from 0-1,
which is a normalized sliding scale of the color ramp.}

\item{end}{The end color that the base color should fade into.}

\item{mid}{For pre-made diverging palettes, the color of the midpoint of the
scale.}
}
\value{
\code{\link[=make_color_pal]{make_color_pal()}} returns a function that accepts a numeric vector
of values between 0 and 1 and returns a character vector of hex color
codes. \code{\link[=ramp_blue]{ramp_blue()}}, \code{\link[=ramp_yellow]{ramp_yellow()}}, \code{\link[=ramp_yelblu]{ramp_yelblu()}}, and \code{\link[=ramp_orgblu]{ramp_orgblu()}}
return a character vector of hex color codes.
}
\description{
These color ramp functions create color scales that can be used for making
ggplot2 plots and gt tables. Color ramps are based on the \link{palette_wjake}
color palette.
}
\details{
\code{\link[=make_color_pal]{make_color_pal()}} can be used to create a color ramp function for any set of
valid colors.

\code{\link[=ramp_blue]{ramp_blue()}}, \code{\link[=ramp_yellow]{ramp_yellow()}}, and \code{\link[=ramp_yelblu]{ramp_yelblu()}} are pre-made color ramps
based on the blue and yellow colors from the \link{palette_wjake} color palette.

\code{\link[=ramp_orgblu]{ramp_orgblu()}} is a pre-made color ramp based on the first two colors from
the \link{palette_okabeito} color palette.
}
\examples{
new_ramp <- make_color_pal(c("red", "grey", "blue"))
new_ramp(seq(0, 1, length.out = 10))

# Pre-made palettes
ramp_blue(seq(0, 1, by = 0.2))
ramp_yellow(seq(0.2, 1, length.out = 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fmt-text.R
\name{formatting}
\alias{formatting}
\alias{fmt_digits}
\alias{fmt_count}
\alias{fmt_corr}
\alias{fmt_prop}
\alias{fmt_pct}
\alias{fmt_prop_pct}
\title{Text and Number Formatting}
\usage{
fmt_digits(
  x,
  digits = 1,
  big_mark = ",",
  min_value = -Inf,
  max_value = Inf,
  sub_threshold = 1/(10^digits),
  keep_boundary = FALSE,
  ...
)

fmt_count(x, ...)

fmt_corr(x, digits = 3, ...)

fmt_prop(x, digits = 2, ...)

fmt_pct(x, digits = 0, ...)

fmt_prop_pct(x, digits = 0, ...)
}
\arguments{
\item{x}{Number to be formatted.}

\item{digits}{Number of decimal places to retain.}

\item{big_mark}{Character used between every 3 digits to separate thousands.}

\item{min_value}{The minimum value \code{x} can take.}

\item{max_value}{The maximum value \code{x} can take.}

\item{sub_threshold}{The threshold to use when replacing value extremely
close to \code{min_value} or \code{max_value}. By default, this is determined by the
\code{digits} specified.}

\item{keep_boundary}{Whether to preserve true values of boundaries (i.e.,
\code{min_value} and \code{max_value}). For example if \code{digits = 3}, \code{min_value = 0},
and \code{keep_boundary = FALSE}, a value exactly equal to 0 will become
"<.001". If \code{keep_boundary = TRUE}, then a value of 0 will remain "0.000",
and other small values (e.g., 0.000001) will continue to be replaced with
"<.001".}

\item{...}{Additional arguments passed to \code{\link[scales:comma]{scales::number_format()}} or
\code{\link[=fmt_digits]{fmt_digits()}}. See Details for additional information.}
}
\value{
The updated character object of the same length as \code{x}.
}
\description{
These formatting functions are used to format numerical values in a
consistent manner. This is useful for printing numbers inline with text, as
well as for formatting tables.
}
\details{
\code{\link[=fmt_digits]{fmt_digits()}} is a wrapper for \code{\link[scales:comma]{scales::number_format()}} and prints a number
with the specified number of digits, suppressing values close to the minimum
and maximum values as necessary.

Several helper functions are provided that wrap \code{\link[=fmt_digits]{fmt_digits()}} with common
patterns of \code{min_value}, \code{max_value}, and \code{digits}.
\itemize{
\item \code{\link[=fmt_count]{fmt_count()}} is used for formatting integer values. Prints whole numbers
with no decimals.
\item \code{\link[=fmt_corr]{fmt_corr()}} is used to format correlations or similar indices that are
bounded between [-1, 1]. By default, these values report 3 decimal
places, and the leading 0 is removed as required by APA (2020; section
6.36).
\item \code{\link[=fmt_prop]{fmt_prop()}} is used to format proportions or similar indices that are
bounded between [0, 1]. Similar to \code{\link[=fmt_corr]{fmt_corr()}}, leading 0s are removed.
By default, 2 decimal places are reported.
\item \code{\link[=fmt_pct]{fmt_pct()}} is used to format percentage values that are bounded
[0, 100]. By default, no decimal places are reported.
\item \code{\link[=fmt_prop_pct]{fmt_prop_pct()}} formats proportions bounded by [0, 1] as percentages.
That is, we first take \code{x * 100} and then apply \code{\link[=fmt_pct]{fmt_pct()}}.
}
}
\examples{
fmt_digits(runif(5, min = 5, max = 15), digits = 1)
fmt_digits(runif(5, min = 5, max = 15), digits = 3)

fmt_count(sample(10000:99999, size = 5))

fmt_corr(runif(5, min = -1, max = 1))

fmt_prop(runif(5, min = 0, max = 1))

fmt_pct(runif(5, min = 0, max = 100))

fmt_prop_pct(runif(5, min = 0, max = 1))

}
\references{
American Psychological Association. (2020). \emph{Publication manual
of the American Psychological Association} (7th ed.).
\doi{doi:10.1037/0000165-000}
}
\seealso{
Other formatters: 
\code{\link{apa_words}()}
}
\concept{formatters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rounding.R
\name{round_to}
\alias{round_to}
\title{Round to a specified value}
\usage{
round_to(x, accuracy, direction = c("nearest", "up", "down", "random"))
}
\arguments{
\item{x}{A numeric value to round.}

\item{accuracy}{The accuracy with which to round (i.e., round to the nearest
\code{accuracy}).}

\item{direction}{The direction to round. \code{nearest} (default) will round to
the nearest \code{accuracy}, \code{up} will round up to the nearest \code{accuracy},
\code{down} will round down to the nearest \code{accuracy}, and \code{random} will
randomly round up or down to the nearest \code{accuracy} as described by
Matthews & Harel (2011).}
}
\value{
A numeric rounded to the specified accuracy.
}
\description{
Round to a specified value
}
\examples{
round_to(15, accuracy = 7, direction = "nearest")
round_to(15, accuracy = 7, direction = "up")
round_to(20, accuracy = 7, direction = "down")
}
\references{
Matthews, G., & Harel, O. (2011). Data confidentiality: A review
of methods for statistical disclosure limitation and methods for assessing
privacy. \emph{Statistics Surveys, 5}, 1--29. \doi{10.1214/11-SS074}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-scale-palette.R
\name{scale_palette}
\alias{scale_palette}
\title{Palette color scale}
\arguments{
\item{palette}{A character vector of the color palette to use in the scale
(e.g., \link{palette_wjake}, \link{palette_okabeito}).}

\item{order}{Numeric vector listing the order in which the colors should be
used. Default is the order of the palette vector.}

\item{darken}{Relative amount by which the scale should be darkened (for
positive values) or lightened (for negative values).}

\item{alpha}{Alpha transparency level of the color. Default is no
transparency.}

\item{...}{common discrete scale parameters: \code{name}, \code{breaks}, \code{labels},
\code{na.value}, \code{limits}, \code{guide}, and \code{aesthetics}. See \link[ggplot2:discrete_scale]{ggplot2::discrete_scale} for
more details.}
}
\value{
A \code{ScaleDiscrete} object that can be added to a \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
This is a qualitative scale using the color palette used for
https://wjakethompson.com. See \link{palette_wjake} for details.
}
\examples{
library(ggplot2)

ggplot(penguins, aes(x = bill_len, y = flipper_len, color = species)) +
  geom_point() +
  scale_color_wjake()

ggplot(penguins, aes(bill_len, fill = species)) +
  geom_density(alpha = 0.7) +
  scale_fill_okabeito(order = c(1, 2, 5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-theme-wjake.R
\name{theme_wjake}
\alias{theme_wjake}
\title{Personalized ggplot2 theme}
\usage{
theme_wjake(
  base_size = 11.5,
  base_family = "Source Sans Pro",
  header_family = NULL,
  base_line_size = base_size/22,
  base_rect_size = base_size/22,
  ink = "black",
  paper = "white",
  accent = "#FED766",
  continuous = ramp_blue(c(0.1, 1), end = "#FFFFFF"),
  discrete = palette_wjake,
  transparent = FALSE,
  ...
)
}
\arguments{
\item{base_size}{base font size, given in pts.}

\item{base_family}{base font family}

\item{header_family}{font family for titles and headers. The default, \code{NULL},
uses theme inheritance to set the font. This setting affects axis titles,
legend titles, the plot title and tag text.}

\item{base_line_size}{base size for line elements}

\item{base_rect_size}{base size for rect elements}

\item{ink, paper, accent}{colour for foreground, background, and accented elements respectively.}

\item{continuous}{A character vector of valid colors that will be
interpolated into a continuous color scale.}

\item{discrete}{A character vector of colors to use for discrete color
scales.}

\item{transparent}{Logical indicator for whether the background of the plot
should be transparent.}

\item{...}{Additional parameters passed to \code{\link[ggplot2:theme]{ggplot2::theme()}}.}
}
\value{
A \code{theme} object that can be added to a \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.
}
\description{
Based on \code{\link[ggplot2:ggtheme]{ggplot2::theme_minimal()}}.
}
\examples{
library(ggplot2)

ggplot(penguins, aes(x = bill_len, y = flipper_len)) +
  geom_point(aes(color = species), na.rm = TRUE) +
  labs(
    x = "Bill length (mm)",
    y = "Flipper length (mm)",
    title = "Seminal ggplot2 scatterplot example",
    subtitle = "A plot that is only useful for demonstration purposes",
    caption = "Brought to you by the letter *p*",
    color = "Species"
  ) +
  theme_wjake(base_family = "sans")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gt-theme-apa.R
\name{gt_theme_apa}
\alias{gt_theme_apa}
\title{Title}
\usage{
gt_theme_apa(
  data,
  dec_dig = 1,
  corr_dig = 3,
  fmt_extreme = TRUE,
  bg_color = "#FFFFFF",
  font = "Arial",
  font_size = 11,
  ...
)
}
\arguments{
\item{data}{The gt table data object.}

\item{dec_dig}{The number of decimal places to round to for numeric values.}

\item{corr_dig}{The number of decimal places to round to for numeric values
all constrained to be between [-1, 1].}

\item{fmt_extreme}{Logical indicator for whether values close to extremes
should be suppressed (e.g., .000001 becomes <.001).}

\item{bg_color}{The background color of the table.}

\item{font}{The font to use for the table.}

\item{font_size}{The base font size for the table.}

\item{...}{Additional options passed to \code{\link[gt:tab_options]{gt::tab_options()}}}
}
\value{
An object of class \code{gt-tbl}.
}
\description{
Title
}
\examples{
gt::gt(head(penguins)) |>
  gt_theme_apa() |>
  gt::fmt_number(year, sep_mark = "", decimals = 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write-bib.R
\name{write_pkg_bib}
\alias{write_pkg_bib}
\title{Write a bibliography for R packages}
\usage{
write_pkg_bib(pkg, file, update = FALSE)
}
\arguments{
\item{pkg}{A vector of packages to create a \code{.bib} file for.}

\item{file}{The file to save the references.}

\item{update}{Should packages be updated before creating the bibliography?}
}
\value{
A list containing the citations. Citations are also written to the
file as a side effect.
}
\description{
Write a bibliography for R packages
}
\examples{
write_pkg_bib(c("ggplot2", "gt"), file = tempfile(fileext = "bib"))
}
